@@ -28,16 +28,36 @@ class Intersect;
 // particular direction), and is used to calculate how light scatters
 // from the surface.
 //
-// The Bsdf may be reference-counted using Ref<Bsdf>.  However Bsdf
-// objects are not deleted when the reference-count goes to zero, only
-// the destructor is called.  This is because Bsdf objects are not
-// allocated using the standard allocator.
+// Because Bsdf objects are allocated extremely often, they are
+// allocated using a special memory-arena, and only freed in bulk
+// later on.  Morever, their destructor is never called.
+//
+// So subclasses of Bsdf should:
+//
+//  (1) Only allocate memory using the Intersect object pointed to by
+//      Bsdf::isec as an arena, i.e., using "new (isec) ...".  Note
+//      that this means STL classes should not be used without a
+//      custom allocator.
+//
+//  (2) Not depend on their destructor being called, as it usually
+//      won't be.  In practice this means they should never declare a
+//      destructor.
 //
 class Bsdf
 {
 public:
 
   enum Flags {
+    //
+    // BSDF sample classification flags:  These classify BSDF samples
+    // into a number of different categories.
+    //
+    // They are used both descriptively, e.g., in the
+    // Bsdf::Sample::flags field, and as input arguments for various
+    // Bsdf methods (e.g. Bsdf::sample and Bsdf::eval), for describing
+    // what sorts of samples are to be considered.
+    //
+
     // Sample directions: reflection, or transmission (through the surface)
     //
     REFLECTIVE		= 0x10,
@@ -56,7 +76,27 @@ public:
     ALL_LAYERS		= SPECULAR|GLOSSY|DIFFUSE,
 
     // Mask of all flags
-    ALL			= ALL_DIRECTIONS|ALL_LAYERS
+    ALL			= ALL_DIRECTIONS|ALL_LAYERS,
+
+
+    //
+    // Informational BSDF flags: these give extra information about a
+    // BSDF sample.  They are only used descriptively.
+    //
+
+    // This is an output-only flag that says a sample comes from
+    // "translucency" -- basically this is a special type of
+    // SPECULAR+TRANSMISSIVE sample corresponding to a non-zero
+    // transmittance return value from Material::transmittance /
+    // Surface::occludes / Scene::occludes (typically resulting from
+    // use of alpha/opacity in a material).
+    //
+    // Rendering methods that use Scene::occludes to do shadow-testing
+    // in calculating direct illumination may have to handle samples
+    // with the Bsdf::TRANSLUCENT flag set specially, in order to
+    // avoid double-counting of light from such rays.
+    //
+    TRANSLUCENT		= 0x08
   };
 
   struct Sample
@@ -112,19 +152,9 @@ public:
     float pdf;
   };
 
-  Bsdf (const Intersect &_isec) : isec (_isec), ref_count (0) { }
+  Bsdf (const Intersect &_isec) : isec (_isec) { }
   virtual ~Bsdf () {}
 
-  // Methods implementing the reference-counting protocol used by Ref<>.
-  //
-  // Unlike the RefCounted class most reference-counted classes use as a
-  // superclass, Bsdf objects are not deleted when the reference-count
-  // goes to zero, only the destructor is called.  This is because Bsdf
-  // objects are not allocated using the standard allocator.
-  //
-  void ref () const { ++ref_count; }
-  void deref () const { if (--ref_count <= 0) this->~Bsdf (); }
-
   // Return a sample of this BSDF, based on the parameter PARAM.
   // FLAGS is the types of samples we'd like.
   //
@@ -152,12 +182,6 @@ public:
   // The intersection where this Bsdf was created.
   //
   const Intersect &isec;
-
-private:
-
-  // Reference-count.
-  //
-  mutable int ref_count;
 };
 
 
