@@ -59,7 +59,7 @@ Camera::rotate (const Xform &rot_xform)
   if (rot_xform.reverses_handedness ())
     handedness_reversed = !handedness_reversed;
 
-  _point (forward * rot_xform, up * rot_xform);
+  _point (rot_xform (forward), rot_xform (up));
 }
 
 // Apply XFORM with the target at the origin, then move target back to
@@ -78,7 +78,7 @@ Camera::orbit (const Xform &xform)
   total *= xform;
   total.translate (-target);
 
-  pos *= total;
+  pos.transform (total);
 
   rotate (rot);
 }
@@ -90,7 +90,7 @@ Camera::orbit (const Xform &xform)
 void
 Camera::transform (const Xform &xform)
 {
-  pos *= xform;
+  pos.transform (xform);
   rotate (xform);
 }
 
@@ -130,7 +130,7 @@ Camera::eye_ray (const UV &film_loc, const UV &focus_param) const
       // simulate depth-of-field.
       //
       dist_t src_perturb_x, src_perturb_y;
-      sample_disk (aperture_radius, focus_param, src_perturb_x, src_perturb_y);
+      disk_sample (aperture_radius, focus_param, src_perturb_x, src_perturb_y);
 
       // The end of the camera-ray pointed to by TARG should be perturbed
       // slightly less than SRC, by a factor of 1 / FOCUS_DISTANCE.
