@@ -160,14 +160,16 @@ DirectIllum::sample_light (const Intersect &isec, const Light *light,
 	  //
 	  dist_t max_dist = lsamp.dist ? lsamp.dist - min_dist : scene.horizon;
  
-	  ShadowRay ray (isec, isec.normal_frame.from (lsamp.dir),
-			 min_dist, max_dist);
- 
-	  if (! scene.intersects (ray, context))
+	  Ray ray (isec.normal_frame.origin,
+		   isec.normal_frame.from (lsamp.dir),
+		   min_dist, max_dist);
+
+	  Color transmittance = 1;
+	  if (! scene.occludes (ray, isec.media.medium, transmittance, context))
 	    {
 	      // The sample is not occluded, calculate the actual radiance.
 
-	      Color lsamp_radiance = lsamp.val;
+	      Color lsamp_radiance = lsamp.val * transmittance;
 
 	      lsamp_radiance
 		*= context.volume_integ->transmittance (ray, isec.media.medium);
@@ -229,14 +231,17 @@ DirectIllum::sample_light (const Intersect &isec, const Light *light,
 	      dist_t max_dist
 		= lval.dist ? lval.dist - min_dist : scene.horizon;
 
-	      ShadowRay ray (isec, isec.normal_frame.from (bsamp.dir),
-			     min_dist, max_dist);
+	      Ray ray (isec.normal_frame.origin,
+		       isec.normal_frame.from (bsamp.dir),
+		       min_dist, max_dist);
 
-	      if (! scene.intersects (ray, context))
+	      Color transmittance = 1;
+	      if (! scene.occludes (ray, isec.media.medium, transmittance,
+				    context))
 		{
 		  // The sample is not occluded
 
-		  Color bsamp_radiance = lval.val;
+		  Color bsamp_radiance = lval.val * transmittance;
 
 		  bsamp_radiance
 		    *= context.volume_integ->transmittance (ray,
