@@ -13,8 +13,8 @@
 #include "bbox.h"
 #include "scene.h"
 #include "intersect.h"
-#include "sample-cone.h"
-#include "sample-tangent-disk.h"
+#include "cone-sample.h"
+#include "tangent-disk-sample.h"
 
 #include "far-light.h"
 
@@ -32,7 +32,9 @@ FarLight::sample (const Intersect &isec, const UV &param) const
 {
   // Sample a cone pointing at our light.
   //
-  Vec s_dir = isec.normal_frame.to (frame.from (sample_cone (angle, param)));
+  Vec s_dir
+    = isec.normal_frame.to (frame.from (cone_sample (cos_half_angle, param)));
+  float pdf = cos_half_angle == 1 ? 1 : cone_sample_pdf (cos_half_angle);
 
   if (isec.cos_n (s_dir) > 0 && isec.cos_geom_n (s_dir) > 0)
     return Sample (intensity, pdf, s_dir, 0);
@@ -52,11 +54,12 @@ FarLight::sample (const UV &param, const UV &dir_param) const
   // sample point will appear in the same direction from any location
   // in the scene.
 
-  Vec s_dir = frame.from (sample_cone (angle/2, dir_param));
-  Pos s_pos = sample_tangent_disk (scene_center, scene_radius, s_dir, param);
+  Vec s_dir = frame.from (cone_sample (cos_half_angle, dir_param));
+  Pos s_pos = tangent_disk_sample (scene_center, scene_radius, s_dir, param);
 
   // Adjust pdf to include disk sampling.
   //
+  float pdf = cos_half_angle == 1 ? 1 : cone_sample_pdf (cos_half_angle);
   float s_pdf = pdf / (PIf * scene_radius * scene_radius);
 
   return FreeSample (intensity, s_pdf, s_pos, -s_dir);
@@ -72,12 +75,16 @@ FarLight::sample (const UV &param, const UV &dir_param) const
 Light::Value
 FarLight::eval (const Intersect &isec, const Vec &dir) const
 {
-  Vec light_normal_dir = isec.normal_frame.to (frame.z);
+  if (cos_half_angle < 1)
+    {
+      Vec light_normal_dir = isec.normal_frame.to (frame.z);
+      float pdf = cone_sample_pdf (cos_half_angle);
 
-  if (dot (dir, light_normal_dir) >= min_cos)
-    return Value (intensity, pdf, 0);
-  else
-    return Value ();
+      if (dot (dir, light_normal_dir) >= cos_half_angle)
+	return Value (intensity, pdf, 0);
+    }
+
+  return Value ();
 }
 
 
@@ -87,15 +94,21 @@ FarLight::eval (const Intersect &isec, const Vec &dir) const
 Color
 FarLight::eval_environ (const Vec &dir) const
 {
-  // Cosine of the angle between DIR and the direction of this light.
-  //
-  float cos_light_dir = dot (dir, frame.z);
+  if (cos_half_angle < 1)
+    {
+      // Cosine of the angle between DIR and the direction of this light.
+      //
+      float cos_light_dir = dot (dir, frame.z);
 
-  // If COS_LIGHT_DIR is greater than MIN_COS, then DIR must be within
-  // ANGLE/2 of the light direction, so return the light's color;
-  // otherwise just return 0.
-  //
-  return (cos_light_dir > min_cos) ? intensity : 0;
+      // If COS_LIGHT_DIR is greater than COS_HALF_ANGLE, then DIR must be
+      // within ANGLE/2 of the light direction, so return the light's
+      // color; otherwise just return 0.
+      //
+      if (cos_light_dir > cos_half_angle)
+	return intensity;
+    }
+
+  return 0;
 }
 
 
