@@ -31,9 +31,16 @@ public:
   // ANGLE is the apparent (linear) angle subtended by of the light.
   // INTENSITY is the amount of light emitted per steradian.
   //
-  FarLight (const Vec &_dir, float _angle, const Color &_intensity)
-    : intensity (_intensity), angle (_angle), frame (_dir.unit ()),
-      pdf (1 / angle), min_cos (cos (_angle / 2)),
+  // As a special case, when ANGLE is exactly 0 -- creating an
+  // infinitely-far-away point-light -- then INTENSITY is the
+  // absolute intensity, not the intensity per steradian.
+  //
+  FarLight (const Vec &_dir, float angle, const Color &_intensity)
+    : intensity (_intensity), frame (_dir.unit ()),
+      // To ensure that if ANGLE is zero, COS_HALF_ANGLE becomes
+      // exactly 1 (because we explicitly compare with that), don't
+      // trust the cos function to return exactly the right result.
+      cos_half_angle (angle == 0 ? 1 : cos (angle / 2)),
       scene_radius (0)
   { }
 
@@ -58,6 +65,10 @@ public:
   //
   virtual bool is_environ_light () const { return true; }
 
+  // Return true if this is a point light.
+  //
+  virtual bool is_point_light () const { return cos_half_angle == 1; }
+
   // Evaluate this environmental light in direction DIR (in world-coordinates).
   //
   virtual Color eval_environ (const Vec &dir) const;
@@ -68,7 +79,6 @@ public:
   virtual void scene_setup (const Scene &scene);
 
   Color intensity;
-  dist_t angle;
 
 private:
   
@@ -76,17 +86,9 @@ private:
   //
   Frame frame;
 
-  // As our light subtends a constant angle, and we sample it uniformly
-  // by solid angle, we have a constant pdf.
+  // The cosine of half the angle subtended by this light's cone.
   //
-  const float pdf;
-
-  // The minimum cosine of the angle between a sample and this light.
-  // Any samples where the cosine is less than this (meaning the angle
-  // between the sample and the light direction is greater) do not hit
-  // the light.
-  //
-  dist_t min_cos;
+  float cos_half_angle;
 
   // Center and radius of a bounding sphere for the entire scene.
   //
