@@ -15,13 +15,19 @@
 
 #include "pos.h"
 #include "vec.h"
+#include "xform-base.h"
 
 
 namespace snogray {
 
 
-// A frame is a coordinate-system basis, plus a position.
-// Positions and vectors may be converted to or from the frame.
+// A "frame of reference", relative to the world frame, consisting of
+// a coordinate-system basis plus an origin.  Positions and vectors
+// may be converted to or from the frame (from or to the world-frame,
+// respectively).  The origin is only used when converting positions.
+//
+// A frame is basically equivalent to a transformation matrix in
+// functionality, but in many cases is a nicer abstraction.
 //
 class Frame
 {
@@ -45,6 +51,16 @@ public:
     : origin (0,0,0), x (_x), y (_y), z (_z)
     { }
 
+  // A frame corresponding to the transformation matrix FRAME_TO_WORLD.
+  //
+  Frame (const Matrix4<dist_t> &frame_to_world)
+    : origin (
+	 frame_to_world (3, 0), frame_to_world (3, 1), frame_to_world (3, 2)),
+      x (frame_to_world (0, 0), frame_to_world (0, 1), frame_to_world (0, 2)),
+      y (frame_to_world (1, 0), frame_to_world (1, 1), frame_to_world (1, 2)),
+      z (frame_to_world (2, 0), frame_to_world (2, 1), frame_to_world (2, 2))
+  { }
+
   // A frame with the given _Z basis vector; the other basis vectors are
   // chosen automatically (no guarantee is made about them except that
   // they will be orthogonal to _Z).
@@ -86,6 +102,23 @@ public:
   //
   Pos from (const Pos &pos) const { return origin + from (Vec (pos)); }
 
+  // Return this frame transformed by XFORM.
+  //
+  Frame transformed (const XformBase<dist_t> &xform) const
+  {
+    return Frame (xform (origin), xform (x), xform (y), xform (z));
+  }
+
+  // Transform this frame by XFORM.
+  //
+  void transform (const XformBase<dist_t> &xform)
+  {
+    origin.transform (xform);
+    x.transform (xform);
+    y.transform (xform);
+    z.transform (xform);
+  }
+
   // The "position" of the frame; used when converting positions.
   //
   Pos origin;
