@@ -27,20 +27,25 @@ using namespace snogray;
 
 
 
+// Create an ImageOutput object for writing to FILENAME, with a size of
+// WIDTH, HEIGHT.  PARAMS holds any additional optional parameters.
+//
 ImageOutput::ImageOutput (const std::string &filename,
 			  unsigned _width, unsigned _height,
 			  const ValTable &params)
   : width (_width), height (_height),
-    min_y (0),
-    sink (ImageSink::open (filename, _width, _height, params)),
-    filter_conv (params),
     intensity_scale (params.get_float ("exposure", 1)),
-    intensity_power (params.get_float ("contrast", 1))
+    intensity_power (params.get_float ("contrast", 1)),
+    min_y (0),
+    sample_base_x (params.get_float ("sample-base-x", 0)),
+    sample_base_y (params.get_float ("sample-base-y", 0)),
+    sink (ImageSink::open (filename, _width, _height, params)),
+    filter_conv (params)
 {
 }
 
 void
-ImageOutput::_set_min_y (int new_min_y)
+ImageOutput::set_raw_min_y (int new_min_y)
 {
   ASSERT (new_min_y >= min_y);
 
@@ -64,18 +69,22 @@ ImageOutput::_set_min_y (int new_min_y)
       for (unsigned x = 0; x < width; x++)
 	{
 	  Tint pixel = r->pixels[x];
+	  Color col = pixel.alpha_scaled_color ();
+	  Tint::alpha_t alpha = pixel.alpha;
 
 	  float weight = r->weights[x];
 	  if (weight > 0)
-	    pixel /= weight;
+	    {
+	      col *= 1 / weight;
+	      alpha *= 1 / weight;
+	    }
 
 	  if (intensity_scale != 1)
-	    pixel *= intensity_scale;
+	    col *= intensity_scale;
 	  if (intensity_power != 1)
-	    pixel = Tint (pow (pixel.unscaled_color(), intensity_power),
-			  pixel.alpha);
+	    col = pow (max (col, 0.f), intensity_power);
 
-	  r->pixels[x] = pixel;
+	  r->pixels[x] = Tint (col, alpha);
 	}
 
       sink->write_row (r->pixels);
@@ -90,7 +99,7 @@ ImageOutput::~ImageOutput ()
 {
   // Write as-yet unwritten rows
   //
-  _set_min_y (height);
+  set_raw_min_y (height);
   flush ();
 }
 
@@ -126,6 +135,8 @@ ImageOutput::_row (int y)
 //
 void ImageOutput::add_sample (float sx, float sy, const Tint &tint)
 {
+  sx -= sample_base_x;
+  sy -= sample_base_y;
   filter_conv.add_sample (sx, sy, tint, *this);
 }
 
