@@ -44,7 +44,11 @@ public:
     std::vector<float> weights;
   };
 
-  ImageOutput (const std::string &filename, unsigned _width, unsigned _height,
+  // Create an ImageOutput object for writing to FILENAME, with a size
+  // of WIDTH, HEIGHT.  PARAMS holds any additional optional parameters.
+  //
+  ImageOutput (const std::string &filename,
+	       unsigned width, unsigned height,
 	       const ValTable &params = ValTable::NONE);
   ~ImageOutput ();
 
@@ -68,15 +72,29 @@ public:
   bool has_alpha_channel () const { return sink->has_alpha_channel (); }
 
   // Flush any buffered rows until the current minimum (buffered) row is
-  // ImageOutput::min_y.
+  // ImageOutput::min_y.  NEW_MIN_Y is in the sample coordinate-system,
+  // not the output coordinate-system.
   //
-  void set_min_y (int new_min_y)
+  void set_min_sample_y (int new_min_y)
   {
-    // Set the raw min_y leaving some room for the filter support.
+    // Set the raw min_y leaving some room for the filter support,
+    // and converting between the sample coordinate-system and the
+    // output-image coordinate-system.
     //
-    _set_min_y (max (min_y, new_min_y - int (filter_radius ())));
+    new_min_y -= int (sample_base_y);
+    new_min_y -= int (filter_radius ());
+    set_raw_min_y (max (min_y, new_min_y));
   }
 
+  // Flush any buffered rows until the current minimum (buffered) row is
+  // ImageOutput::min_y.  Unlike ImageOutput::set_min_sample_y, this
+  // directly operates on the buffer, in the coordinate-system of the
+  // output image, and does not add any adjustment for the filter
+  // support or for any offset between the sample and output-image
+  // coordinate-systems.
+  //
+  void set_raw_min_y (int new_min_y);
+
   // Return the number rows or columns on either side of any pixel that are
   // effective when a sample is added inside that pixel (because of filter
   // convolution).
@@ -98,6 +116,8 @@ public:
   }
 
   // Return true if the given X or Y coordinate is valid.
+  // The coordinates are in the output image's coordinate-system
+  // (so in the range 0,0 - WIDTH,HEIGHT).
   //
   // [These methods are callbacks used by Filterconv<ImageOutput>.]
   //
@@ -112,7 +132,7 @@ public:
   SampleRow &row (int y)
   {
     int offs = y - min_y;
-    if (y >= 0 && y < int (rows.size ()))
+    if (offs >= 0 && offs < int (rows.size ()))
       return *rows[offs];
     else
       return _row (y);
@@ -126,20 +146,25 @@ public:
   //
   unsigned width, height;
 
+  // Modifiers applied to output values.  Note that these are applied
+  // when a particular row is flushed to the output file, not while the
+  // row is being accumulated (this usually doesn't matter, but can in
+  // the case of image recovery, where previous _output_ values are
+  // being copied).
+  //
+  float intensity_scale;   // intensity multiplier (1 == nop)
+  float intensity_power;   // power which intensity is raised to (1 == nop)
+
+private:
+
   // Row number of first row buffered in memory.  No row before this can
   // be addressed.
   //
   int min_y;
 
-
-private:
-
-  // Flush any buffered rows until the current minimum (buffered) row is
-  // ImageOutput::min_y.  Unlike ImageOutput::set_min_y, this directly
-  // operates on the buffer, and does not add any adjustment for the
-  // filter support.
+  // Base-coordinates of the sample coordinate-system.
   //
-  void _set_min_y (int new_min_y);
+  float sample_base_x, sample_base_y;
 
   // Internal version of the ImageOutput::row() method which handles
   // rows not in ImageOutput::rows.
@@ -156,9 +181,6 @@ private:
   // ImageOutput::min_y.
   //
   std::deque<SampleRow *> rows;
-
-  float intensity_scale;   // intensity multiplier (1 == nop)
-  float intensity_power;   // power which intensity is raised to (1 == nop)
 };
 
 }
