@@ -1,6 +1,6 @@
 // instance.h -- Transformed virtual instance of a surface
 //
-//  Copyright (C) 2007, 2008, 2009, 2010  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2007, 2008, 2009, 2010, 2011  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -39,7 +39,27 @@ public:
 
   // Return true if this surface intersects RAY.
   //
-  virtual bool intersects (const ShadowRay &ray, RenderContext &context) const;
+  virtual bool intersects (const Ray &ray, RenderContext &context) const;
+
+  // Return true if this surface completely occludes RAY.  If it does
+  // not completely occlude RAY, then return false, and multiply
+  // TOTAL_TRANSMITTANCE by the transmittance of the surface in medium
+  // MEDIUM.
+  //
+  // Note that this method does not try to handle non-trivial forms of
+  // transparency/translucency (for instance, a "glass" material is
+  // probably considered opaque because it changes light direction as
+  // well as transmitting it).
+  //
+  // [This interface is slight awkward for reasons of speed --
+  // returning and checking for a boolean value for common cases is
+  // significantly faster than, for instance, a simple "transmittance"
+  // method, which requires handling Color values for all cases.]
+  //
+  virtual bool occludes (const Ray &ray, const Medium &medium,
+			 Color &total_transmittance,
+			 RenderContext &context)
+    const;
 
   // Return a bounding box for this surface.
   //
@@ -50,7 +70,7 @@ private:
   struct IsecInfo : public Surface::IsecInfo
   {
     IsecInfo (const Ray &ray,
-	      const Instance *_instance,
+	      const Instance &_instance,
 	      const Surface::IsecInfo *_subspace_isec_info)
       : Surface::IsecInfo (ray),
 	instance (_instance),
@@ -61,7 +81,10 @@ private:
 				      RenderContext &context)
       const;
 
-    const Instance *instance;
+    virtual TexCoords tex_coords () const;
+    virtual Vec normal () const;
+
+    const Instance &instance;
 
     const Surface::IsecInfo *subspace_isec_info;
   };
