@@ -13,13 +13,10 @@
 #ifndef __INTEG_H__
 #define __INTEG_H__
 
-#include "ray.h"
-#include "tint.h"
-
 
 namespace snogray {
 
-class Scene;
+class GlobalRenderState;
 class RenderContext;
 
 
@@ -35,10 +32,12 @@ public:
   {
   public:
 
-    GlobalState (const Scene &_scene) : scene (_scene) { }
+    GlobalState (const GlobalRenderState &rstate)
+      : global_render_state (rstate)
+    { }
     virtual ~GlobalState () { } 
 
-    const Scene &scene;
+    const GlobalRenderState &global_render_state;
   };
 
 protected:
