@@ -54,8 +54,6 @@ public:
   //
   Intersect (const Intersect &isec);
 
-  ~Intersect ();
-
   // Return the cosine of the angle between the surface normal and VEC.
   // VEC must be in this intersection's normal frame, and must be
   // normalized.
@@ -150,17 +148,7 @@ public:
 
   // BSDF used at this intersection.
   //
-  Ref<const Bsdf> bsdf;
-
-  // Oft-used properties of surface.
-  //
-  const void *smoothing_group;
-
-  // If non-zero, a surface which cannot shadow itself (for non-planar
-  // objects, this applies to the current side of the surface -- the one
-  // corresponding to the normal N).
-  //
-  const Surface *no_self_shadowing;
+  const Bsdf *bsdf;
 
   // Texture coordinates for this intersection.
   //
