@@ -142,7 +142,7 @@ public:
 
   // Evaluate this environmental light in direction DIR (in world-coordinates).
   //
-  virtual Color eval_environ (const Vec &/*dir*/) const { return 0; };
+  virtual Color eval_environ (const Vec &/*dir*/) const { return 0; }
 
   // Do any scene-related setup for this light.  This is is called once
   // after the entire scene has been loaded.
