@@ -29,7 +29,7 @@
 #include "excepts.h"
 #include "scene.h"
 #include "mirror.h"
-#include "plastic.h"
+#include "thin-glass.h"
 #include "sphere.h"
 #include "mesh.h"
 #include "scene.h"
@@ -37,7 +37,6 @@
 #include "phong.h"
 #include "lambert.h"
 #include "cook-torrance.h"
-#include "sphere-light.h"
 #include "material-dict.h"
 #include "glow.h"
 
@@ -173,15 +172,16 @@ TdsLoader::convert_material (Lib3dsMaterial *m)
 
   if (m->transparency > 0)
     //
-    // The plastic material we use for transparency has no real color;
-    // it only transmits light, or reflects due to Fresnel reflection.
+    // The thin_glass material we use for transparency has no real
+    // color; it only transmits light, or reflects due to Fresnel
+    // reflection.
     //
     // We use the plastic's index of refraction to try to control
     // shininess: a shininess of 0 means an IOR of 1, so no Fresnel
-    // reflection from the surface; a shininess of 1 means an IOR of 2,
-    // which should provide some nice reflections.
+    // reflection from the surface; a shininess of 1 means an IOR of
+    // 2, which should provide some nice reflections.
     //
-    mat = new Plastic (m->transparency, 1 + m->shininess);
+    mat = new ThinGlass (m->transparency, 1 + m->shininess);
   else
     {
       Color diffuse = color (m->diffuse);
@@ -365,7 +365,7 @@ TdsLoader::set_camera (Camera &camera, Lib3dsCamera *c, const Xform &xform)
 {
   Vec up (0, 0, 1);
 
-  up *= Xform::z_rotation (c->roll * PI / 180);
+  up.transform (Xform::z_rotation (c->roll * PI / 180));
 
   Xform dir_xform = xform.inverse().transpose();
 
@@ -373,13 +373,13 @@ TdsLoader::set_camera (Camera &camera, Lib3dsCamera *c, const Xform &xform)
 //   cout << "   orig pos:   " << pos (c->position) << endl;
 //   cout << "   orig targ:  " << pos (c->target) << endl;
 //   cout << "   orig up:    " << up << endl;
-//   cout << "   xform pos:  " << pos (c->position) * xform << endl;
-//   cout << "   xform targ: " << pos (c->target) * xform << endl;
-//   cout << "   xform up:   " << up * dir_xform << endl;
+//   cout << "   xform pos:  " << xform (pos (c->position)) << endl;
+//   cout << "   xform targ: " << xform (pos (c->target))  << endl;
+//   cout << "   xform up:   " << dir_xform (up)  << endl;
 
   camera.set_vert_fov (c->fov * PI / 180);
-  camera.move (pos (c->position) * xform);
-  camera.point (pos (c->target) * xform, up * dir_xform);
+  camera.move (xform (pos (c->position)));
+  camera.point (xform (pos (c->target)), dir_xform (up));
 }
 
 
@@ -462,7 +462,7 @@ TdsLoader::convert (Lib3dsNode *node, const Xform &xform,
 	  lib3ds_matrix_mul (X, N, M);
 #endif
 
-	  Xform vert_xform = Xform (X) * xform;
+	  Xform vert_xform = xform (Xform (X));
 
 	  // Get the actual mesh.
 	  //
@@ -569,8 +569,8 @@ TdsLoader::convert (Lib3dsNode *node, const Xform &xform,
 		  if (! vert_info[vi].used)
 		    {
 		      vert_info[vi].index
-			= mesh->add_vertex (pos (m->pointL[f->points[i]])
-					    * vert_xform);
+			= mesh->add_vertex (
+				  vert_xform (pos (m->pointL[f->points[i]])));
 		      vert_info[vi].used = true;
 		    }
 
@@ -624,10 +624,9 @@ TdsLoader::convert (const Xform &xform)
 	if (! (l->object_flags & LIB3DS_OBJECT_HIDDEN))
 #endif
 	{
-	  const Pos loc = pos (l->position) * xform;
+	  const Pos loc = xform (pos (l->position));
 	  const Color intens = color (l->color) * l->multiplier * area_scale;
 
-	  scene->add (new SphereLight (loc, radius, intens));
 	  scene->add (new Sphere (new Glow (intens), loc, radius));
 	}
     }
