@@ -1,6 +1,6 @@
 // load-ply.h -- Load a .ply format mesh file
 //
-//  Copyright (C) 2006, 2007, 2008  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2006, 2007, 2008, 2010  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -23,12 +23,9 @@ namespace snogray {
 class Mesh;
 class Material;
 
-// Load mesh from a .ply format mesh file into MESH.  If non-zero the
-// material MAT will be used for triangles loaded (otherwise MESH's
-// default material will be used).
+// Load mesh from a .ply format mesh file into MESH.
 //
-extern void load_ply_file (const std::string &filename, Mesh &mesh,
-			   const Ref<const Material> &mat = 0);
+extern void load_ply_file (const std::string &filename, Mesh &mesh);
 
 }
 
