@@ -1,6 +1,6 @@
 // mesh.cc -- Mesh surface			-*- coding: utf-8 -*-
 //
-//  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -16,7 +16,6 @@
 #include "excepts.h"
 #include "string-funs.h"
 
-#include "shadow-ray.h"
 #include "tripar-isec.h"
 
 #include "mesh.h"
@@ -176,6 +175,37 @@ Mesh::add_vertices (const std::vector<MPos> &new_verts)
   return base_vert;
 }
 
+// Add all the positions described by NEW_VERTS as vertices in this
+// mesh.  NEW_VERTS should contain three elements for each vertex, to
+// be used as the x, y, and z coordinates of the vertex.
+//
+// The index in the mesh of the first of the new vertices is returned;
+// it should be passed to any subsequent calls to Mesh::add_triangles
+// or Mesh::add_normals.
+//
+Mesh::vert_index_t
+Mesh::add_vertices (const std::vector<scoord_t> &new_verts)
+{
+  vert_index_t base_vert = vertices.size ();
+  unsigned num_new_verts = new_verts.size () / 3;
+
+  // Note that the Pos default constructor does not initialize the Pos
+  // members, so the following will grow the Mesh::vertices vector,
+  // but not give the new elements any value.
+  //
+  vertices.resize (base_vert + num_new_verts);
+
+  unsigned nvi = 0;
+  for (unsigned v = base_vert; v < base_vert + num_new_verts; v++)
+    {
+      vertices[v]
+	= Pos (new_verts[nvi], new_verts[nvi + 1], new_verts[nvi + 2]);
+      nvi += 3;
+    }
+
+  return base_vert;
+}
+
 // Add all the normal vectors in NEW_NORMALS as vertex normals in this
 // mesh, corresponding to all the vertices starting from BASE_VERT
 // (which should be a value returned from an earlier call to
@@ -197,6 +227,44 @@ Mesh::add_normals (const std::vector<MVec> &new_normals, vert_index_t base_vert)
 			 new_normals.begin(), new_normals.end());
 }
 
+// Add all the normal vectors described by NEW_NORMALS as vertex
+// normals in this mesh, corresponding to all the vertices starting
+// from BASE_VERT (which should be a value returned from an earlier
+// call to Mesh::add_vertices).
+//
+// NEW_NORMALS should contain three elements for each normal, to be
+// used as the x, y, and z components of the normal.
+//
+void
+Mesh::add_normals (const std::vector<sdist_t> &new_normals,
+		   vert_index_t base_vert)
+{
+  unsigned num_new_normals = new_normals.size () / 3;
+
+  // Not sure what to do if normals after BASE_VERT already exist, of
+  // if vertices before BASE_VERT don't have normals yet, so just barf
+  // in those cases.
+  //
+  if (base_vert != vertex_normals.size ())
+    throw runtime_error ("BASE_VERT incorrect in Mesh::add_normals");
+  if (base_vert + num_new_normals != vertices.size ())
+    throw runtime_error ("Size of NEW_NORMALS incorrect in Mesh::add_normals");
+
+  // Note that the Pos default constructor does not initialize the Pos
+  // members, so the following will grow the Mesh::normals vector,
+  // but not give the new elements any value.
+  //
+  vertex_normals.resize (base_vert + num_new_normals);
+
+  unsigned nni = 0;
+  for (unsigned ni = base_vert; ni < base_vert + num_new_normals; ni++)
+    {
+      vertex_normals[ni]
+	= Vec (new_normals[nni], new_normals[nni + 1], new_normals[nni + 2]);
+      nni += 3;
+    }
+}
+
 // Add all the UV values in NEW_UVS as vertex UV values in this mesh,
 // corresponding to all the vertices starting from BASE_VERT (which
 // should be a value returned from an earlier call to
@@ -217,6 +285,41 @@ Mesh::add_uvs (const std::vector<UV> &new_uvs, vert_index_t base_vert)
   vertex_uvs.insert (vertex_uvs.end(), new_uvs.begin(), new_uvs.end());
 }
 
+// Add all the UV values described by NEW_UVS as vertex UV values in
+// this mesh, corresponding to all the vertices starting from
+// BASE_VERT (which should be a value returned from an earlier call
+// to Mesh::add_vertices).
+//
+// NEW_UVS should contain two elements for each UV.
+//
+void
+Mesh::add_uvs (const std::vector<float> &new_uvs, vert_index_t base_vert)
+{
+  unsigned num_new_uvs = new_uvs.size () / 2;
+
+  // Not sure what to do if uvs after BASE_VERT already exist, of
+  // if vertices before BASE_VERT don't have uvs yet, so just barf
+  // in those cases.
+  //
+  if (base_vert != vertex_uvs.size ())
+    throw runtime_error ("BASE_VERT incorrect in Mesh::add_uvs");
+  if (base_vert + num_new_uvs != vertices.size ())
+    throw runtime_error ("Size of NEW_UVS incorrect in Mesh::add_uvs");
+
+  // Note that the UV default constructor does not initialize the UV
+  // members, so the following will grow the Mesh::uvs vector, but not
+  // give the new elements any value.
+  //
+  vertex_uvs.resize (base_vert + num_new_uvs);
+
+  unsigned nui = 0;
+  for (unsigned ui = base_vert; ui < base_vert + num_new_uvs; ui++)
+    {
+      vertex_uvs[ui] = UV (new_uvs[nui], new_uvs[nui + 1]);
+      nui += 2;
+    }
+}
+
 // Add new triangles to the mesh using vertices from TRI_VERT_INDICES.
 // TRI_VERT_INDICES should contain three entries for each new triangle;
 // the indices in TRI_VERT_INDICES are relative to BASE_VERT (which
@@ -258,10 +361,10 @@ Mesh::Triangle::intersect (Ray &ray, RenderContext &context) const
   Vec edge1 = v(1) - corner, edge2 = v(2) - corner;
 
   dist_t t, u, v;
-  if (triangle_intersect (corner, edge1, edge2, ray, t, u, v))
+  if (triangle_intersects (corner, edge1, edge2, ray, t, u, v))
     {
       ray.t1 = t;
-      return new (context) IsecInfo (ray, this, u, v);
+      return new (context) IsecInfo (ray, *this, u, v);
     }
 
   return 0;
@@ -273,7 +376,7 @@ Mesh::Triangle::intersect (Ray &ray, RenderContext &context) const
 Frame
 Mesh::Triangle::IsecInfo::make_frame (const Pos &origin, const Vec &norm) const
 {
-  const Mesh &mesh = triangle->mesh;
+  const Mesh &mesh = triangle.mesh;
 
   // The usual value is NORM x AXIS, where AXIS is an arbitrary axis
   // vector.  This yields a value for S that's pointing "around" AXIS,
@@ -318,59 +421,45 @@ Intersect
 Mesh::Triangle::IsecInfo::make_intersect (const Media &media, RenderContext &context)
   const
 {
-  Vec norm = triangle->raw_normal ();
-
   // Point of intersection.
   //
   Pos point = ray.end ();
 
   // Our geometric frame uses the real surface geometry.
   //
-  Frame geom_frame = make_frame (point, triangle->raw_normal ());
+  Frame geom_frame = make_frame (point, triangle.raw_normal ());
 
   // Calculate the normal frame; if the mesh contains vertex normal
   // information, calculate it by interpolating our vertex normals,,
   // otherwise just copy the geometric frame.
   //
   Frame normal_frame;
-  if (! triangle->mesh.vertex_normals.empty ())
+  if (! triangle.mesh.vertex_normals.empty ())
     {
-      Vec norm = triangle->vnorm(0) * (1 - u - v);
-      norm += triangle->vnorm(1) * u;
-      norm += triangle->vnorm(2) * v;
+      Vec norm = triangle.vnorm(0) * (1 - u - v);
+      norm += triangle.vnorm(1) * u;
+      norm += triangle.vnorm(2) * v;
       norm = norm.unit ();	// normalize
+
+      // Make the normal consistent with the geometry.  Since the
+      // geometric normal depends on the handedness of the mesh, which
+      // is easy to get wrong, we assume the interpolated ("shading")
+      // normal is probably the correct one, and flip the geometric
+      // normal.
+      //
+      if (geom_frame.to (norm).z < 0)
+	geom_frame.z = -geom_frame.z;
+
       normal_frame = make_frame (point, norm);
     }
   else
     normal_frame = geom_frame;
 
-  // Texture-coordinates for the three vertices of the triangle.
-  //
-  // If this mesh doesn't have per-vertex UV values, then they are just the
-  // raw barycentric coordinates of the vertices.
-  //
-  UV T0, T1, T2;
-  if (triangle->mesh.vertex_uvs.empty ())
-    {
-      T0 = UV (0, 0);
-      T1 = UV (1, 0);
-      T2 = UV (0, 1);
-    }
-  else
-    {
-      T0 = triangle->vuv (0);
-      T1 = triangle->vuv (1);
-      T2 = triangle->vuv (2);
-    }
-
-  // Change in UV values for edge1 and edge2 of the triangle.
-  //
-  UV dTdu = T1 - T0;
-  UV dTdv = T2 - T0;
-
-  // Texture-coordinates for POINT (as opposed to the "raw" triangle UV
-  // value in the variables "u" and "v").
+  // Calculate 2d texture-coordinates for POINT (as opposed to the
+  // "raw" triangle UV value in the variables "u" and "v").
   //
+  UV T0, dTdu, dTdv;
+  triangle.get_texture_params (T0, dTdu, dTdv);
   UV T = T0 + dTdu * u + dTdv * v;
 
   //
@@ -405,19 +494,21 @@ Mesh::Triangle::IsecInfo::make_intersect (const Media &media, RenderContext &con
 
   // Edge coordinate deltas in normal space.
   //
-  Vec e1_w = triangle->v(1) - triangle->v(0); // triangle edge 1 in world space
-  Vec e2_w = triangle->v(2) - triangle->v(0); // triangle edge 2 in world space
+  Vec e1_w = triangle.v(1) - triangle.v(0); // triangle edge 1 in world space
+  Vec e2_w = triangle.v(2) - triangle.v(0); // triangle edge 2 in world space
   Vec e1 = normal_frame.to (e1_w);	      // edge 1 in normal space
   Vec e2 = normal_frame.to (e2_w);	      // edge 2 in normal space
 
   // Calculate du/ds, du/dt, dv/ds, and dv/dt.
   //
   float du_den = e1.x * (e2.y + e2.z) - e2.x * (e1.y + e1.z);
-  float duds = (e2.y + e2.z) / du_den;
-  float dudt = -e2.x / du_den;
+  float inv_du_den = du_den == 0 ? 0 : 1 / du_den;
+  float duds = (e2.y + e2.z) * inv_du_den;
+  float dudt = -e2.x * inv_du_den;
   float dv_den = e2.x * (e1.y + e1.z) - e1.x * (e2.y + e2.z);
-  float dvds = (e1.y + e1.z) / dv_den;
-  float dvdt = -e1.x / dv_den;
+  float inv_dv_den = dv_den == 0 ? 0 : 1 / dv_den;
+  float dvds = (e1.y + e1.z) * inv_dv_den;
+  float dvdt = -e1.x * inv_dv_den;
 
   // Calculate texture coordinate partial derivatives, in normal space.
   //
@@ -426,19 +517,34 @@ Mesh::Triangle::IsecInfo::make_intersect (const Media &media, RenderContext &con
 
   // Make the intersect object.
   //
-  Intersect isec (ray, media, context, *triangle->mesh.material,
-		  normal_frame, geom_frame, T, dTds,dTdt);
+  return Intersect (ray, media, context, *triangle.mesh.material,
+		    normal_frame, geom_frame, T, dTds,dTdt);
+}
 
-  isec.no_self_shadowing = triangle;
-  isec.smoothing_group = static_cast<const void *>(&triangle->mesh);
+// Return the texture-coordinates of this intersection.
+//
+TexCoords
+Mesh::Triangle::IsecInfo::tex_coords () const
+{
+  UV T0, dTdu, dTdv;
+  triangle.get_texture_params (T0, dTdu, dTdv);
+  UV T = T0 + dTdu * u + dTdv * v;
+  return TexCoords (ray.end(), T);
+}
 
-  return isec;
+// Return the normal of this intersection (in the world frame).
+//
+Vec
+Mesh::Triangle::IsecInfo::normal () const
+{
+  // XXX is geometric normal enough?
+  return triangle.raw_normal ();
 }
 
 // Return true if this surface intersects RAY.
 //
 bool
-Mesh::Triangle::intersects (const ShadowRay &ray, RenderContext &) const
+Mesh::Triangle::intersects (const Ray &ray, RenderContext &) const
 {
   // We have to convert the types to match that of RAY first.
   //
@@ -446,58 +552,44 @@ Mesh::Triangle::intersects (const ShadowRay &ray, RenderContext &) const
   Vec edge1 = v(1) - corner, edge2 = v(2) - corner;
 
   dist_t t, u, v;
-  if (triangle_intersect (corner, edge1, edge2, ray, t, u, v))
+  return triangle_intersects (corner, edge1, edge2, ray, t, u, v);
+}
+
+// Return true if this surface completely occludes RAY.  If it does
+// not completely occlude RAY, then return false, and multiply
+// TOTAL_TRANSMITTANCE by the transmittance of the surface in medium
+// MEDIUM.
+//
+// Note that this method does not try to handle non-trivial forms of
+// transparency/translucency (for instance, a "glass" material is
+// probably considered opaque because it changes light direction as
+// well as transmitting it).
+//
+// [This interface is slight awkward for reasons of speed --
+// returning and checking for a boolean value for common cases is
+// significantly faster than, for instance, a simple "transmittance"
+// method, which requires handling Color values for all cases.]
+//
+bool
+Mesh::Triangle::occludes (const Ray &ray, const Medium &medium,
+			  Color &total_transmittance,
+			  RenderContext &)
+  const
+{
+  // We have to convert the types to match that of RAY first.
+  //
+  Pos corner = v(0);
+  Vec edge1 = v(1) - corner, edge2 = v(2) - corner;
+
+  dist_t t, u, v;
+  if (triangle_intersects (corner, edge1, edge2, ray, t, u, v))
     {
-#if 1
-      if (ray.isec.smoothing_group == static_cast<const void *>(&mesh))
-	{    
-	  bool real_back = dot (raw_normal_unscaled(), ray.dir) > 0;
+      // avoid calculating texture coords if possible
+      if (mesh.material->fully_occluding ())
+	return true;
 
-	  // We only get suspicious about the validity of the shadow if RAY is
-	  // coming from a difference surface when we compare the virtual
-	  // smoothed normal with the real surface normal.
-	  //
-	  if (real_back != ray.isec.back)
-	    {
-	      // XXX to get the other triangle, we take advantage of the
-	      // fact that Mesh::Triangle::IsecInfo::make_intersect sets
-	      // Intersect::no_self_shadowing to the triangle.
-	      //
-	      const Triangle *other_tri
-		= static_cast<const Triangle *>(ray.isec.no_self_shadowing);
-	      bool other_back
-		= dot (other_tri->raw_normal_unscaled(), ray.dir) > 0;
-
-	      if (real_back != other_back)
-		return false;
-
-#if 0
-	      // Vertex indices of this and the other triangle
-	      //
-	      vert_index_t v0i = vi[0];
-	      vert_index_t v1i = vi[1];
-	      vert_index_t v2i = vi[2];
-	      vert_index_t ov0i = other_tri->vi[0];
-	      vert_index_t ov1i = other_tri->vi[1];
-	      vert_index_t ov2i = other_tri->vi[2];
-
-	      // We check to see if this triangle shares any vertices with
-	      // the other triangle; if so, then if the source intersection
-	      // was on the front/back of OTHER_TRI, we reject shadows if
-	      // RAY hit the same surface (front/back) of this triangle.
-	      // Doing this eliminates the most common false shadowing
-	      // cases in smooth meshes.
-
-	      if (v0i == ov0i || v0i == ov1i || v0i == ov2i
-		  || v1i == ov0i || v1i == ov1i || v1i == ov2i
-		  || v2i == ov0i || v2i == ov1i || v2i == ov2i)
-		return ray.isec.back == other_back;
-#endif
-	    }
-	}
-#endif
-
-      return true;
+      IsecInfo isec_info (Ray (ray, t), *this, u, v);
+      return mesh.material->occludes (isec_info, medium, total_transmittance);
     }
 
   return false;
@@ -726,7 +818,7 @@ Mesh::transform (const Xform &xform)
   const SXform xf = SXform (xform);
 
   for (vert_index_t v = 0; v < vertices.size (); v++)
-    vertices[v] *= xf;
+    vertices[v].transform (xf);
 
   if (vertex_normals.size() > 0)
     {
@@ -736,7 +828,7 @@ Mesh::transform (const Xform &xform)
       SXform norm_xf = xf.inverse().transpose();
 
       for (vert_index_t v = 0; v < vertex_normals.size (); v++)
-	vertex_normals[v] *= norm_xf;
+	vertex_normals[v].transform (norm_xf);
     }
 
   if (xform.reverses_handedness ())
