@@ -1,6 +1,6 @@
 // mesh.h -- Mesh surface
 //
-//  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -92,6 +92,16 @@ public:
   //
   vert_index_t add_vertices (const std::vector<MPos> &new_verts);
 
+  // Add all the positions described by NEW_VERTS as vertices in this
+  // mesh.  NEW_VERTS should contain three elements for each vertex,
+  // to be used as the x, y, and z coordinates of the vertex.
+  //
+  // The index in the mesh of the first of the new vertices is
+  // returned; it should be passed to any subsequent calls to
+  // Mesh::add_triangles or Mesh::add_normals.
+  //
+  vert_index_t add_vertices (const std::vector<scoord_t> &new_verts);
+
   // Add all the normal vectors in NEW_NORMALS as vertex normals in this
   // mesh, corresponding to all the vertices starting from BASE_VERT
   // (which should be a value returned from an earlier call to
@@ -100,6 +110,17 @@ public:
   void add_normals (const std::vector<MVec> &new_normals,
 		    vert_index_t base_vert);
 
+  // Add all the normal vectors described by NEW_NORMALS as vertex
+  // normals in this mesh, corresponding to all the vertices starting
+  // from BASE_VERT (which should be a value returned from an earlier
+  // call to Mesh::add_vertices).
+  //
+  // NEW_NORMALS should contain three elements for each normal, to be
+  // used as the x, y, and z components of the normal.
+  //
+  void add_normals (const std::vector<sdist_t> &new_normals,
+		    vert_index_t base_vert);
+
   // Add all the UV values in NEW_UVS as vertex UV values in this mesh,
   // corresponding to all the vertices starting from BASE_VERT (which
   // should be a value returned from an earlier call to
@@ -107,6 +128,15 @@ public:
   //
   void add_uvs (const std::vector<UV> &new_uvs, vert_index_t base_vert);
 
+  // Add all the UV values described by NEW_UVS as vertex UV values in
+  // this mesh, corresponding to all the vertices starting from
+  // BASE_VERT (which should be a value returned from an earlier call
+  // to Mesh::add_vertices).
+  //
+  // NEW_UVS should contain two elements for each UV.
+  //
+  void add_uvs (const std::vector<float> &new_uvs, vert_index_t base_vert);
+
   // Add new triangles to the mesh using vertices from TRI_VERT_INDICES.
   // TRI_VERT_INDICES should contain three entries for each new triangle;
   // the indices in TRI_VERT_INDICES are relative to BASE_VERT (which
@@ -184,101 +214,7 @@ private:
 
   // A single triangle in the mesh.
   //
-  class Triangle : public Surface
-  {
-  public:
-
-    Triangle (const Mesh &_mesh) : mesh (_mesh) { }
-
-    Triangle (const Mesh &_mesh,
-	      vert_index_t v0i, vert_index_t v1i, vert_index_t v2i)
-      : mesh (_mesh)
-    {
-      vi[0] = v0i;
-      vi[1] = v1i;
-      vi[2] = v2i;
-    }
-
-    void operator= (const Triangle &triang)
-    {
-      vi[0] = triang.vi[0];
-      vi[1] = triang.vi[1];
-      vi[2] = triang.vi[2];
-    }
-
-    // If this surface intersects RAY, change RAY's maximum bound (Ray::t1)
-    // to reflect the point of intersection, and return a Surface::IsecInfo
-    // object describing the intersection (which should be allocated using
-    // placement-new with CONTEXT); otherwise return zero.
-    //
-    virtual const IsecInfo *intersect (Ray &ray, RenderContext &context) const;
-
-    // Return true if this surface intersects RAY.
-    //
-    virtual bool intersects (const ShadowRay &ray, RenderContext &context)
-      const;
-
-    // Return a bounding box for this surface.
-    //
-    virtual BBox bbox () const;
-
-    // Vertex NUM of this triangle
-    //
-    Pos v (unsigned num) const { return Pos (mesh.vertices[vi[num]]); }
-
-    // Normal of vertex NUM (assuming this mesh contains vertex normals!)
-    //
-    Vec vnorm (unsigned num) const
-    {
-      return Vec (mesh.vertex_normals[vi[num]]);
-    }
-
-    // UV value of vertex NUM (assuming this mesh contains vertex UV values!)
-    //
-    UV vuv (unsigned num) const
-    {
-      return mesh.vertex_uvs[vi[num]];
-    }
-
-    // These both return the "raw" normal of this triangle, not doing
-    // any normal interpolation.  Note that they return ordinary
-    // double-precision vectors, not the single-precision vectors used
-    // in the mesh (because most uses want the former).
-    //
-    const Vec raw_normal_unscaled () const
-    {
-      Vec e1 = v(1) - v(0), e2 = v(2) - v(0);
-      return mesh.left_handed ? cross (e2, e1) : cross (e1, e2);
-    }
-    const Vec raw_normal () const
-    {
-      return raw_normal_unscaled().unit ();
-    }
-
-    struct IsecInfo : public Surface::IsecInfo
-    {
-      IsecInfo (const Ray &ray, const Triangle *_triangle, dist_t _u, dist_t _v)
-	: Surface::IsecInfo (ray), triangle (_triangle), u (_u), v (_v)
-      { }
-
-      virtual Intersect make_intersect (const Media &media, RenderContext &context)
-	const;
-
-      // Return a normal frame FRAME at ORIGIN, with basis vectors
-      // calculated from the normal NORM.
-      //
-      Frame make_frame (const Pos &orgin, const Vec &norm) const;
-
-      const Triangle *triangle;
-      dist_t u, v;
-    };
-
-    const Mesh &mesh;
-
-    // Indices into mesh vertices array
-    //
-    vert_index_t vi[3];
-  };
+  class Triangle;
 
   // Recalculate this mesh's bounding box.
   //
@@ -324,6 +260,178 @@ public:
 };
 
 
+
+// Mesh::Triangle
+
+// A single triangle in a Mesh.
+//
+class Mesh::Triangle : public Surface
+{
+public:
+
+  friend class Mesh;
+
+  Triangle (const Mesh &_mesh) : mesh (_mesh) { }
+
+  Triangle (const Mesh &_mesh,
+	    vert_index_t v0i, vert_index_t v1i, vert_index_t v2i)
+    : mesh (_mesh)
+  {
+    vi[0] = v0i;
+    vi[1] = v1i;
+    vi[2] = v2i;
+  }
+
+  void operator= (const Triangle &triang)
+  {
+    vi[0] = triang.vi[0];
+    vi[1] = triang.vi[1];
+    vi[2] = triang.vi[2];
+  }
+
+  // If this surface intersects RAY, change RAY's maximum bound (Ray::t1)
+  // to reflect the point of intersection, and return a Surface::IsecInfo
+  // object describing the intersection (which should be allocated using
+  // placement-new with CONTEXT); otherwise return zero.
+  //
+  virtual const IsecInfo *intersect (Ray &ray, RenderContext &context) const;
+
+  // Return true if this surface intersects RAY.
+  //
+  virtual bool intersects (const Ray &ray, RenderContext &context)
+    const;
+
+  // Return true if this surface completely occludes RAY.  If it does
+  // not completely occlude RAY, then return false, and multiply
+  // TOTAL_TRANSMITTANCE by the transmittance of the surface in medium
+  // MEDIUM.
+  //
+  // Note that this method does not try to handle non-trivial forms of
+  // transparency/translucency (for instance, a "glass" material is
+  // probably considered opaque because it changes light direction as
+  // well as transmitting it).
+  //
+  // [This interface is slight awkward for reasons of speed --
+  // returning and checking for a boolean value for common cases is
+  // significantly faster than, for instance, a simple "transmittance"
+  // method, which requires handling Color values for all cases.]
+  //
+  virtual bool occludes (const Ray &ray, const Medium &medium,
+			 Color &total_transmittance,
+			 RenderContext &context)
+    const;
+
+  // Return a bounding box for this surface.
+  //
+  virtual BBox bbox () const;
+
+  // Vertex NUM of this triangle
+  //
+  Pos v (unsigned num) const { return Pos (mesh.vertices[vi[num]]); }
+
+  // Normal of vertex NUM (assuming this mesh contains vertex normals!)
+  //
+  Vec vnorm (unsigned num) const
+  {
+    return Vec (mesh.vertex_normals[vi[num]]);
+  }
+
+  // UV value of vertex NUM (assuming this mesh contains vertex UV values!)
+  //
+  UV vuv (unsigned num) const
+  {
+    return mesh.vertex_uvs[vi[num]];
+  }
+
+  // These both return the "raw" normal of this triangle, not doing
+  // any normal interpolation.  Note that they return ordinary
+  // double-precision vectors, not the single-precision vectors used
+  // in the mesh (because most uses want the former).
+  //
+  const Vec raw_normal_unscaled () const
+  {
+    Vec e1 = v(1) - v(0), e2 = v(2) - v(0);
+    return mesh.left_handed ? cross (e2, e1) : cross (e1, e2);
+  }
+  const Vec raw_normal () const
+  {
+    return raw_normal_unscaled().unit ();
+  }
+
+private:
+
+  // Return 2D texture-coordinate information for this triangle.
+  // The 2D texture-coordinate of vertex 0 (with barycentric
+  // coordinate 0,0) is returned in T0.  The change in 2D
+  // texture-coordinates between vertex 0 and vertex 1 (corresponding
+  // to barycentric coordinate "u") is returned in DTDU, and the
+  // change between vertex 0 and vertex 2 (corresponding to
+  // barycentric coordinate "v") is returned in DTDV.
+  //
+  void get_texture_params (UV &T0, UV &dTdu, UV &dTdv) const
+  {
+    // Texture-coordinates for the three vertices of the triangle.
+    //
+    // If this mesh doesn't have per-vertex UV values, a per-triangle
+    // mapping is used.
+    //
+    UV T1, T2;
+    if (mesh.vertex_uvs.empty ())
+      {
+	// The assignment of UV values to triangle vertices in the
+	// absence of UV-mapping information is fairly arbitrary.
+	//
+	// We just use a mapping compaible with PBRT, where the middle
+	// vertex of a triangle has UV coordinates 0,0, and the first
+	// and last vertices have coordinates 1,0 and 0,1
+	// respectively.
+
+	T0 = UV (1, 0);
+	T1 = UV (0, 0);
+	T2 = UV (0, 1);
+      }
+    else
+      {
+	T0 = vuv (0);
+	T1 = vuv (1);
+	T2 = vuv (2);
+      }
+
+    // Change in UV values for edge1 and edge2 of the triangle.
+    //
+    dTdu = T1 - T0;
+    dTdv = T2 - T0;
+  }
+
+  struct IsecInfo : public Surface::IsecInfo
+  {
+    IsecInfo (const Ray &ray, const Triangle &_triangle, dist_t _u, dist_t _v)
+      : Surface::IsecInfo (ray), triangle (_triangle), u (_u), v (_v)
+    { }
+
+    virtual Intersect make_intersect (const Media &media, RenderContext &context)
+      const;
+
+    virtual TexCoords tex_coords () const;
+    virtual Vec normal () const;
+
+    // Return a normal frame FRAME at ORIGIN, with basis vectors
+    // calculated from the normal NORM.
+    //
+    Frame make_frame (const Pos &orgin, const Vec &norm) const;
+
+    const Triangle &triangle;
+    dist_t u, v;
+  };
+
+  const Mesh &mesh;
+
+  // Indices into mesh vertices array
+  //
+  vert_index_t vi[3];
+};
+
+
 }
 
 #endif /* __MESH_H__ */
