@@ -1,6 +1,6 @@
 // path-integ.cc -- Path-tracing surface integrator
 //
-//  Copyright (C) 2010  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2010, 2011  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -60,9 +60,8 @@ public:
 PathInteg::GlobalState::GlobalState (const GlobalRenderState &rstate,
 				     const ValTable &params)
   : SurfaceInteg::GlobalState (rstate),
-    min_path_len (params.get_uint ("min-len", 5)),
-    russian_roulette_terminate_probability (
-      params.get_float ("rr-term-prob,rr-term", 0.5f)),
+    min_path_len (params.get_uint ("min-len", 3)),
+    max_path_len (params.get_uint ("max-len", 25)),
     direct_illum (
       params.get_uint ("direct-samples,dir-samples,dir-samps",
 		       rstate.params.get_uint ("light-samples", 1))),
@@ -323,15 +322,28 @@ PathInteg::Li (const Ray &ray, const Media &orig_media,
       if (bsdf_samp.pdf == 0 || bsdf_samp.val == 0)
 	break;
 
+      // Add this BSDF sample to PATH_TRANSMITTANCE.
+      //
+      path_transmittance
+	*= bsdf_samp.val * abs (isec.cos_n (bsdf_samp.dir)) / bsdf_samp.pdf;
+
       // If this path is getting long, use russian roulette to randomly
       // terminate it.
       //
       if (path_len > global.min_path_len)
 	{
-	  float rr_term_prob = global.russian_roulette_terminate_probability;
+	  // RR_CONTINUE_PROB is the probability that we'll continue
+	  // this path.
+	  //
+	  // We make it proportional to the current path transmittance
+	  // so that paths with high-transmittance, which have a
+	  // bigger effect on the final result, will be explored
+	  // farther.
+	  //
+	  float rr_continue_prob = min (1.f, path_transmittance.intensity ());
 	  float russian_roulette = context.random ();
-
-	  if (russian_roulette < rr_term_prob)
+	  
+	  if (russian_roulette > rr_continue_prob)
 	    //
 	    // Terminated!
 	    break;
@@ -339,18 +351,15 @@ PathInteg::Li (const Ray &ray, const Media &orig_media,
 	    // Don't terminate.  Adjust PATH_TRANSMITTANCE to reflect
 	    // the fact that we tried.
 	    //
-	    // By dividing by the probability of termination, which is
+	    // By dividing by the probability of continuation, which is
 	    // less than 1, we boost the intensity of paths that survive
 	    // russian-roulette, which will exactly compensate for the
 	    // zero value of paths that are terminated by it.
 	    //
-	    path_transmittance /= 1 - rr_term_prob;
+	    path_transmittance /= rr_continue_prob;
 	}
-
-      // Add this BSDF sample to PATH_TRANSMITTANCE.
-      //
-      path_transmittance
-	*= bsdf_samp.val * abs (isec.cos_n (bsdf_samp.dir)) / bsdf_samp.pdf;
+      if (path_len == global.max_path_len)
+	break;
 
       // Update ISEC_RAY to point from ISEC's position in the direction
       // of the BSDF sample.  
@@ -359,9 +368,21 @@ PathInteg::Li (const Ray &ray, const Media &orig_media,
 		      isec.normal_frame.from (bsdf_samp.dir),
 		      min_dist, scene.horizon);
 
-      // Remember whether we followed a specular sample.
+      // Remember whether we followed a specular sample, because such
+      // samples are normally not accounted for in the direct-lighting
+      // term, and so if the sample hits an emitter, the emitter
+      // should be included (normally emission terms are ignored
+      // because their contribution is accounted for by the preceding
+      // direct-lighting term).
       //
-      after_specular_sample = (bsdf_samp.flags & Bsdf::SPECULAR);
+      // We "don't count" Bsdf::TRANSLUCENT for this purpose (even
+      // though they're nominally "specular"), because they actually
+      // _are_ accounted for in the direct-lighting term.
+      //
+      after_specular_sample
+	= ((bsdf_samp.flags & Bsdf::SPECULAR)
+	   && !((bsdf_samp.flags & Bsdf::TRANSLUCENT)
+		&& path_len > 0));
 
       // If we just followed a refractive (transmissive) sample, we need
       // to update our stack of Media entries:  entering a refractive
