@@ -52,10 +52,9 @@ public:
     //
     unsigned min_path_len;
 
-    // Probability we will terminate a path at each vertex beyond the
-    // MIN_PATH_LEN.
+    // Path-length at which we just give up and return 0.
     //
-    float russian_roulette_terminate_probability;
+    unsigned max_path_len;
 
     // Global state for DirectIllum objects.
     //
