@@ -19,6 +19,7 @@
 #include "image-io.h"
 #include "string-funs.h"
 #include "cmdlineparser.h"
+#include "load.h"
 #if USE_LUA
 # include "load-lua.h"
 #endif
@@ -67,7 +68,7 @@ SceneDef::load (Scene &scene, Camera &camera)
   // Set background (this is done before reading in the scene, so the scene
   // defining code can adjust for the presence of an environment map).
   //
-  string bg_spec = params.get_string ("background");
+  string bg_spec = params.get_string ("scene.background");
   if (! bg_spec.empty ())
     {
       string fmt = strip_prefix (bg_spec, ":");
@@ -98,21 +99,16 @@ SceneDef::load (Scene &scene, Camera &camera)
        spec != specs.end(); spec++)
     try
       {
-	scene.load (spec->name, spec->scene_fmt, camera);
+	load_file (spec->name, spec->scene_fmt, scene, camera, params);
       }
     catch (runtime_error &err)
       {
-	throw runtime_error (spec->user_name + ": Error reading scene: "
-			     + err.what ());
+	throw runtime_error (spec->user_name + ": " + err.what ());
       }
 
   // Cleanup Lua loader state if necessary.
   //
-  // Note that we can't do this if swig doesn't support the "disown"
-  // feature, as without that, scene objects will still be considered
-  // "owned" by swig, and deallocated if we cleanup Lua state.
-  //
-#if USE_LUA && HAVE_SWIG_DISOWN
+#if USE_LUA
   cleanup_load_lua_state ();
 #endif
 }
