@@ -13,13 +13,12 @@
 #ifndef __SCENE_DEF_H__
 #define __SCENE_DEF_H__
 
-#include <iostream>
+#include <iosfwd>
 #include <iomanip>
 
 #include "val-table.h"
 
 #define SCENE_DEF_OPTIONS_HELP "\
- Scene options:\n\
   -b, --background=BG        Use BG as a background and light-source;\n\
                                BG may be a color or the name of an\n\
                                environment-map image file\n\
@@ -39,7 +38,7 @@
 
 #define SCENE_DEF_OPTION_CASES(clp, scene_def)				\
   case 'b':								\
-    scene_def.params.set ("background", clp.opt_arg ());		\
+    scene_def.params.set ("scene.background", clp.opt_arg ());		\
     break;								\
 									\
   case 'I':								\
