@@ -1,6 +1,6 @@
 // snogassert.cc -- Simple sanity checking
 //
-//  Copyright (C) 2009, 2010  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2009, 2010, 2011  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -21,12 +21,12 @@ void
 snogray::assert_fail (const char *failure_msg,
 		      const char *src_file, unsigned src_line)
 {
-  std::string msg;
+  std::string msg ("assertion failure: ");
 
   if (src_file)
     {
       msg += src_file;
-      msg += ": ";
+      msg += ":";
       msg += stringify (src_line);
       msg += ": ";
     }
