@@ -1,6 +1,6 @@
 // snogray.cc -- Main driver for snogray ray tracer
 //
-//  Copyright (C) 2005, 2006, 2007, 2008, 2010  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2005, 2006, 2007, 2008, 2010, 2011  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -27,6 +27,7 @@
 #include "rusage.h"
 #include "string-funs.h"
 #include "file-funs.h"
+#include "num-cores.h"
 #include "progress.h"
 #include "globals.h"
 
@@ -183,43 +184,7 @@ LimitSpec::apply (CmdLineParser &clp, unsigned range, unsigned base) const
 }
 
 
-// Parsers for --size and --limit command-line option arguments
-
-// Parse a size option argument.  If both a width and height are
-// specified WIDTH and HEIGHT are set.  If only a single number is
-// specified, SIZE is set instead.
-//
-static void
-parse_size_opt_arg (CmdLineParser &clp,
-		    unsigned &width, unsigned &height, unsigned &size)
-{
-  const char *arg = clp.opt_arg ();
-  char *end = 0;
-
-  unsigned num = strtoul (arg, &end, 10);
-
-  if (end && end != arg)
-    {
-      // If no height is given, it will be set according to the camera's
-      // aspect ratio
-      //
-      if (*end == '\0') 
-	{
-	  size = num;
-	  return;
-	}
-
-      arg = end + strspn (end, " ,x");
-
-      width = num;
-      height = strtoul (arg, &end, 10);
-
-      if (end && end != arg && *end == '\0')
-	return;
-    }
-
-  clp.opt_err ("requires a size specification (WIDTHxHEIGHT, or SIZE)");
-}
+// Parser for the --limit command-line option argument
 
 static void
 parse_limit_opt_arg (CmdLineParser &clp,
@@ -279,12 +244,7 @@ help (CmdLineParser &clp, ostream &os)
   os <<
   "Ray-trace an image"
 n
-s "  -s, --size=WIDTHxHEIGHT    Set image size to WIDTH x HEIGHT pixels/lines"
-s "  -w, --width=WIDTH          Set image width to WIDTH pixels"
-s "  -h, --height=HEIGHT        Set image height to HEIGHT lines"
-#if USE_THREADS
-s "  -j, --threads=NUM          Use NUM threads for rendering"
-#endif
+s " Rendering options:"
 n
 s RENDER_OPTIONS_HELP
 n
@@ -307,13 +267,29 @@ s "                               u SIZE      Set scene unit to SIZE, in mm"
 s "                               h           Set camera orientation to horizontal"
 s "                               v           Set camera orientation to vertic"
 n
+s " Scene options:"
+n
 s SCENE_DEF_OPTIONS_HELP
 n
+s " Output image options:"
+n
 s IMAGE_OUTPUT_OPTIONS_HELP
 n
-s "  -C, --continue             Continue a previously aborted render"
+s "  -L, --limit=X,Y+W,H        Limit output to area X,Y - X+W,Y+H"
+s "  -L, --limit=X1,Y1-X2,Y2    Limit output to area X1,Y1 - X2,Y2"
+s "                               Limit coordinates/sizes can be percentages"
+s "                               (with \"%\") or fractions (with \".\") of the"
+s "                               \"nominal\" output image, or integer numbers"
+s "                               of pixels (note: the -s/--size option always"
+s "                               specifies the size of the nominal image)"
 n
-s "  -L, --limit=LIMIT_SPEC     Limit output to area defined by LIMIT_SPEC"
+s " Misc options:"
+n
+#if USE_THREADS
+s "  -j, --threads=NUM          Use NUM threads for rendering (default all cores)"
+n
+#endif
+s "  -C, --continue             Continue a previously aborted render"
 n
 s "  -q, --quiet                Do not output informational or progress messages"
 s "  -P, --no-progress          Do not output progress indicator"
@@ -332,9 +308,6 @@ int main (int argc, char *const *argv)
   // Command-line option specs
   //
   static struct option long_options[] = {
-    { "size",		required_argument, 0, 's' },
-    { "width",		required_argument, 0, 'w' },
-    { "height",		required_argument, 0, 'h' },
     { "limit",		required_argument, 0, 'L' },
     { "quiet",		no_argument,	   0, 'q' },
     { "progress",	no_argument,	   0, 'p' },
@@ -353,7 +326,7 @@ int main (int argc, char *const *argv)
   };
   //
   char short_options[] =
-    "s:w:h:l:qpPCc:"
+    "L:qpPCc:"
 #if USE_THREADS
     "j:"
 #endif
@@ -367,42 +340,23 @@ int main (int argc, char *const *argv)
 
   // Parameters set from the command line
   //
-  unsigned width = 0, height = 0, size = DEFAULT_IMAGE_SIZE;
   LimitSpec limit_x_spec ("min-x", 0), limit_y_spec ("min-y", 0);
   LimitSpec limit_max_x_spec ("max-x", 1.0), limit_max_y_spec ("max-y", 1.0);
-  unsigned num_threads = 1;
+  unsigned num_threads = 0;	// autodetect
   bool recover = false;
   Progress::Verbosity verbosity = Progress::CHATTY;
   bool progress_set = false;
-  ValTable image_params, render_params;
+  ValTable output_params, render_params;
   SceneDef scene_def;
   std::string camera_cmds;	// User commands for the camera
 
 
-  // This speeds up I/O on cin/cout by not syncing with C stdio.
-  //
-  // [Note that in some implementations this leaks about 100KB of memory.]
-  //
-  ios::sync_with_stdio (false);
-
-
   // Parse command-line options
   //
   int opt;
   while ((opt = clp.get_opt ()) > 0)
     switch (opt)
       {
-	// Size options
-	//
-      case 's':
-	parse_size_opt_arg (clp, width, height, size);
-	break;
-      case 'w':
-	width = clp.unsigned_opt_arg ();
-	break;
-      case 'h':
-	height = clp.unsigned_opt_arg ();
-	break;
       case 'L':
 	parse_limit_opt_arg (clp, limit_x_spec, limit_y_spec,
 			     limit_max_x_spec, limit_max_y_spec);
@@ -448,7 +402,7 @@ int main (int argc, char *const *argv)
 
 	// Image options
 	//
-	IMAGE_OUTPUT_OPTION_CASES (clp, image_params);
+	IMAGE_OUTPUT_OPTION_CASES (clp, output_params);
 
 	// Generic options
 	//
@@ -468,7 +422,7 @@ int main (int argc, char *const *argv)
 
   // Output filename
   //
-  string file_name = clp.get_arg();
+  output_params.set ("filename", clp.get_arg ());
 
 
   // Start of "overall elapsed" time
@@ -487,6 +441,11 @@ int main (int argc, char *const *argv)
   Scene scene;
   Camera camera;
 
+  // Height/width of output image (may not be set yet).
+  //
+  unsigned width = output_params.get_uint ("width", 0);
+  unsigned height = output_params.get_uint ("height", 0);
+
   // If the user specified both a width and a height, set the camera aspect
   // ratio to maintain pixels with a 1:1 aspect ratio.  We do this before
   // loading the scene so that scene camera manipulation commands can see
@@ -495,11 +454,24 @@ int main (int argc, char *const *argv)
   if (width && height)
     camera.set_aspect_ratio (float (width) / float (height));
 
-  // Read in the scene/camera definitions
+  // Read in the scene/camera definitions.  We copy our parameter
+  // tables into SCENE_DEF's parameter table with appropriate
+  // prefixes, so that they're available for scene-loaders to
+  // examine/change.
   //
+  scene_def.params.import (render_params, "render.");
+  scene_def.params.import (output_params, "output.");
   CMDLINEPARSER_CATCH (clp, scene_def.load (scene, camera));
 
-  // Do post-load scene setup (nothign can be added to scene after this).
+  // Now rewrite our parameter tables appropriately-prefixed
+  // parameters from SCENE_DEF.  As we earlier stored them, that means
+  // they'll be the same unless the scene-loader changed something.
+  //
+  render_params = scene_def.params.filter_by_prefix ("render.");
+  output_params = scene_def.params.filter_by_prefix ("output.");
+
+
+  // Do post-load scene setup (nothing can be added to scene after this).
   //
   Octree::BuilderFactory octree_builder_factory;
   scene.setup (octree_builder_factory);
@@ -520,33 +492,21 @@ int main (int argc, char *const *argv)
   //
   enable_fp_exceptions ();
 
-  // If the user specified both a width and a height, set the camera aspect
-  // ratio _again_ to maintain pixels with a 1:1 aspect ratio, just in case
-  // the scene file tried to muck with it.
+
+  // Get the final image size.
   //
-  if (width && height)
-    camera.set_aspect_ratio (float (width) / float (height));
-  else
-    // Otherwise, the output image size was not fully specified, so use
-    // the camera's aspect ration to deduce the missing dimension from
-    // the provided one, or from SIZE, which sets the longest dimension
-    // rather than a specific one.
-    {
-      float ar = camera.aspect_ratio ();
+  get_image_size (output_params, camera.aspect_ratio (), DEFAULT_IMAGE_SIZE,
+		  width, height);
 
-      if (!width && !height)
-	{
-	  if (ar > 1)
-	    width = size;
-	  else
-	    height = size;
-	}
+  // Set the camera's aspect ration to be consistent with the final
+  // output image size.
+  //
+  camera.set_aspect_ratio (float (width) / float (height));
 
-      if (width)
-	height = unsigned (width / ar);
-      else
-	width = unsigned (height * ar);
-    }
+
+  // Output filename
+  //
+  std::string file_name = output_params.get_string ("filename");
 
 
   // Set our drawing limits based on the scene size
@@ -593,11 +553,18 @@ int main (int argc, char *const *argv)
 	}
     }
 
+  // Set the base-coordinates of the "output sample space" to LIMIT_X, LIMIT_Y.
+  //
+  if (limit_x != 0)
+    output_params.set ("sample-base-x", limit_x);
+  if (limit_y != 0)
+    output_params.set ("sample-base-y", limit_y);
+
   // If the scene has a non-default background alpha set, then make sure
   // there's an alpha-channel in the output image.
   //
   if (render_params.get_float ("background-alpha", 1) != 1)
-    image_params.set ("alpha-channel", true);
+    output_params.set ("alpha-channel", true);
 
   // Create output image.  The size of what we output is the same as the
   // limit (which defaults to, but is not the same as the nominal output
@@ -607,9 +574,9 @@ int main (int argc, char *const *argv)
   // with IMAGE_SINK_PARAMS or a problem creating the output image, so
   // we create the image before printing any normal output.
   //
-  ImageOutput output (file_name, limit_width, limit_height, image_params);
+  ImageOutput output (file_name, limit_width, limit_height, output_params);
 
-  if (image_params.get_bool ("alpha-channel,alpha")
+  if (output_params.get_bool ("alpha-channel,alpha")
       && !output.has_alpha_channel())
     {
       std::cerr << clp.err_pfx() << file_name
@@ -651,7 +618,7 @@ int main (int argc, char *const *argv)
       cout << "* camera: at " << camera.pos
 	   << ", pointing at "
 	   << (camera.pos + camera.forward * camera.target_dist)
-	   << " (up = " << camera.up << ")"
+	   << " (up = " << camera.up << ", right = " << camera.right << ")"
 	   << endl;
       if (camera.aperture)
 	cout << "* camera: aperture " << camera.aperture
@@ -660,6 +627,16 @@ int main (int argc, char *const *argv)
     }
 
 
+  // If the user didn't specify how many threads to use, try to use as
+  // many as there are CPU cores.
+  //
+  if (num_threads == 0)
+    num_threads = num_cores (1);
+
+  if (num_threads != 1)
+    std::cout << "* using " << num_threads << " threads" << std::endl;
+
+
   // Setup rendering state.  This can take a _very_ long time, because
   // some rendering methods do length pre-rendering computation, so time
   // it.
