@@ -206,6 +206,8 @@ end
 pos = raw.Pos
 vec = raw.Vec
 bbox = raw.BBox
+ray = raw.Ray
+uv = raw.UV
 
 -- Handy scene origin position.
 --
@@ -371,7 +373,7 @@ local function color_tex_val (val)
 end
 
 -- Return TEX, which should be a texture, converted into a
--- floating-poing texture if it isn't one already.
+-- floating-point texture if it isn't one already.
 --
 local function float_tex (tex)
    if is_color_tex (tex) then
@@ -472,9 +474,23 @@ end
 --
 local function postproc_material (mat, params)
    if type (params) == 'table' then
+      -- bump-mapping
+      --
       local bump = params.bump_map or params.bump
-      if bump then mat.bump_map = float_tex (bump) end
+      -- we ignore scalar bump maps, as they have no effect
+      if bump and type (bump) ~= 'number' then
+	 mat.bump_map = float_tex (bump)
+      end
+
+      -- opacity (alpha transparency)
+      --
+      local opacity = params.opacity or params.alpha
+      -- a simple 1 or color(1) means "fully opaque", so can be ignored
+      if opacity and opacity ~= 1 and opacity ~= color(1) then
+	 mat = stencil (opacity, mat)
+      end
    end
+
    return mat
 end   
 
@@ -581,14 +597,14 @@ function glass (params)
    return postproc_material (raw.glass (raw.Medium (_ior, _absorb)), params)
 end
 
--- Return a plastic material.
+-- Return a thin_glass material.
 -- PARAMS can be:
 --   IOR
 --   {ior=IOR, color=COLOR}
 --   {COLOR, ior=IOR}
 -- etc
 --
-function plastic (params)
+function thin_glass (params)
    local _ior = 1.5
    local _color = white
 
@@ -605,7 +621,7 @@ function plastic (params)
 
    _color = color (_color)
 
-   return postproc_material (raw.plastic (_color, _ior), params)
+   return postproc_material (raw.thin_glass (_color, _ior), params)
 end
 
 function glow (col, underlying)
@@ -621,6 +637,10 @@ function norm_glow (intens)
    return raw.norm_glow (intens or 1)
 end   
 
+function stencil (opacity, underlying)
+   opacity = color_tex_val (opacity)
+   return raw.stencil (opacity, underlying)
+end
 
 ----------------------------------------------------------------
 --
@@ -657,7 +677,22 @@ end
 
 -- Make a transform.
 --
-xform = raw.Xform
+function xform (...)
+   --
+   -- SWIG has bugs in handling overloads in conjunction with
+   -- table-to-array conversion (it correctly generates code to do the
+   -- conversion, but the type-checking code for distinguishing the
+   -- various overload cases checks the wrong type), so we use a
+   -- separate helper function "raw_xform" if the argument is a table
+   -- [of transform matrix elements].
+   --
+   local nargs = select ('#', ...)
+   if nargs == 1 and type (select (1, ...)) == 'table' then
+      return raw.raw_xform (...)
+   else
+      return raw.Xform (...)
+   end
+end
 
 identity_xform = raw.Xform_identity
 
@@ -673,6 +708,7 @@ rotate = raw.Xform_rotation
 rotate_x = raw.Xform_x_rotation
 rotate_y = raw.Xform_y_rotation
 rotate_z = raw.Xform_z_rotation
+basis_xform = raw.Xform_basis
 
 -- ... and some abbreviations for them (a bit silly, but composed
 -- transforms can get rather long...).
@@ -789,7 +825,7 @@ function normalize_xform (surf)
    local center = midpoint (bbox.max, bbox.min)
    local max_size = bbox:max_size ()
 
-   return translate (-center.x, -center.y, -center.z) * scale (2 / max_size)
+   return scale (2 / max_size) (translate (-center.x, -center.y, -center.z))
 end
 
 -- Return a transform which will warp SURF to be in a 2x2x2 box centered
@@ -804,8 +840,7 @@ function y_base_normalize_xform (surf)
    local size = bbox.max - bbox.min
    local max_size = bbox:max_size ()
 
-   return translate (-center.x, size.y / 2 - center.y, -center.z)
-      * scale (2 / max_size)
+   return scale (2 / max_size) (translate (-center.x, size.y / 2 - center.y, -center.z))
 end
 
 -- Resize a mesh to fit in a 1x1x1 box, centered at the origin (but with
@@ -813,7 +848,7 @@ end
 --
 function normalize (mesh, xf)
    local norm = y_base_normalize_xform (mesh)
-   if xf then norm = norm * xf end
+   if xf then norm = xf (norm) end
    mesh:transform (norm)
    return mesh
 end
@@ -869,10 +904,10 @@ function solid_cylinder (mat, arg1, ...)
       local xform = arg1
       local emat1, emat2 = select (1, ...), select (2, ...)
 
-      local base = pos(0,0,-1)*xform
-      local axis = vec(0,0,2)*xform
-      local r1 = vec(1,0,0)*xform
-      local r2 = vec(0,1,0)*xform
+      local base = xform (pos (0, 0, -1))
+      local axis = xform (vec (0, 0, 2))
+      local r1 = xform (vec (1, 0, 0))
+      local r2 = xform (vec (0, 1, 0))
 
       return surface_group {
 	 cylinder (mat, xform);
@@ -981,22 +1016,28 @@ end
 --
 -- Lights
 
-function point_light (pos, intens)
-   return raw.PointLight (pos, color (intens))
+function point_light (pos, intens, ...)
+   return raw.PointLight (pos, color (intens), ...)
 end
 
 function sphere_light (pos, radius, intens)
    return raw.SphereLight (pos, radius, color (intens))
 end
 
-function rect_light (corner, side1, side2, intens)
-   return raw.RectLight (corner, side1, side2, color (intens))
+function triangle_light (corner, side1, side2, intens)
+   return raw.TriparLight (corner, side1, side2, false, color (intens))
 end
 
 function far_light (dir, angle, intens)
    return raw.FarLight (dir:unit(), angle, color (intens))
 end
 
+function envmap_light (image_or_filename, ...)
+   if type (image_or_filename) == 'string' then
+      image_or_filename = image (image_or_filename)
+   end
+   return raw.EnvmapLight (raw.envmap (image_or_filename), ...)
+end
 
 ----------------------------------------------------------------
 --
@@ -1042,8 +1083,8 @@ function perturb_pos_tex (src, x, y, z)
 			       float_tex_val (y), float_tex_val (z))
 end
 function perturb_uv_tex (src, u, v)
-   return raw.perturb_pos_tex (tex_val (src),
-			       float_tex_val (u), float_tex_val (v))
+   return raw.perturb_uv_tex (tex_val (src),
+			      float_tex_val (u), float_tex_val (v))
 end
 
 -- Return an interpolation texture, which interpolates between two
@@ -1092,6 +1133,9 @@ local function singleton_tex_fun (name, create)
 end
 
 perlin_tex = singleton_tex_fun ('perlin', raw.perlin_tex)
+perlin_abs_tex
+   = singleton_tex_fun ('perlin_abs',
+			function() return abs_tex (perlin_tex ()) end)
 
 x_tex = singleton_tex_fun ('x', function () return raw.coord_tex(0) end)
 y_tex = singleton_tex_fun ('y', function () return raw.coord_tex(1) end)
@@ -1263,41 +1307,42 @@ function ge_tex (...) return cmp_tex ('GE', ...) end
 
 -- Return a fourier-series summation of SOURCE_TEX, according to PARAMS:
 --
---   Sum[i = 1 to MAX] of (SOURCE_TEX * F_i * Scale(2^(i-1)) / 2^(i-1))
+--   Sum[i = 1 to OCTAVES] of (SOURCE_TEX * F_i * Scale(2^(i-1)) * OMEGA^(i-1))
 --
--- where MAX is the maximum term index to use, F_i are perm-term user
+-- where OCTAVES is the maximum term index to use, F_i are perm-term user
 -- multiplicative factors (defaulting to 1), "... * Scale(2^(i-1))"
 -- means to scale the input coordinates to SOURCE_TEX by 2^(i-1), and
--- "... / 2^(i-1)" means to divide the resulting value of the term by
--- 2^(i-1).
+-- "... * OMEGA^(i-1)" means to multiply the resulting value of the term
+-- by OMEGA^(i-1).
 --
--- If PARAMS is a number, then all F_i are 1, and MAX = PARAMS.
+-- If PARAMS is a number, then all F_i are 1, OMEGA = 0.5, and OCTAVES =
+-- PARAMS.
 --
--- If PARAMS is a table, then F_i is taken from the array entry i in
--- PARAMS (so F_1 is PARAMS[], F_2 is PARAMS[2], etc), and MAX =
--- PARAMS.max, or if there is no "max" entry in PARAMS, then the number
--- of array members of PARAMS.
+-- If PARAMS is a table, then F_i is the value at index i in PARAMS
+-- (F_1 is PARAMS[], F_2 is PARAMS[2], etc), OMEGA = PARAMS.omega or
+-- 0.5 if there is no "omega" in PARAMS, and OCTAVES = PARAMS.octaves,
+-- or the number of array members in PRAMS if there is no "octaves"
+-- entry in PARAMS.
 --
 -- Note that the F_i values do not need to be constants, they can also
 -- be textures.
 --
 -- The result is automatically scaled by the inverse of the constant
--- portions of the sum, Sum[...] (F_i / 2^(i-1)), so that it should have
--- roughly the same magnitude as SOURCE_TEX.  Any non-constant F_i
+-- portions of the sum, Sum[...] (F_i * OMEGA^(i-1)), so that it should
+-- have roughly the same magnitude as SOURCE_TEX.  Any non-constant F_i
 -- values are ignored for the purposes of auto-scaling.
 --
 function fourier_series_tex (source_tex, params)
    local sum = nil
-   local cur_max_pow = 0
    local const_factor_sum = 0
 
-   local function add_term (pow, factor)
+   local function add_term (octave, factor)
       if factor and factor ~= 0 then
-	 local inv_pow_factor = 2^pow
-	 local pow_factor = 1 / inv_pow_factor
+	 local scale_factor = 1.99^octave
+	 local pow_factor = omega ^ octave
 	 local term_factor = factor * pow_factor
 
-	 local term = source_tex * scale (inv_pow_factor)
+	 local term = scale (scale_factor) (source_tex)
 	 if type (term_factor) ~= 'number' or term_factor ~= 1 then
 	    term = term * term_factor
 	 end
@@ -1313,29 +1358,40 @@ function fourier_series_tex (source_tex, params)
 	 else
 	    sum = term
 	 end
-
-	 max_pow = pow
       end
    end
 
-   local max_pow
+   local octaves
+   local cur_octave = 0
    if type (params) == 'table' then
+      omega = params.omega or 0.5
+
+      -- Add terms corresponding to entries in PARAMS.
+      --
       for i, factor in ipairs (params) do
 	 add_term (i-1, factor)
       end
-      max_pow = params.max_pow or params.max
+
+      cur_octave = #params
+      octaves = params.octaves or octaves
    else
-      max_pow = params
+      omega = 0.5
+      octaves = params
    end
 
-   if max_pow then
-      while cur_max_pow <= max_pow do
-	 add_term (cur_max_pow, 1)
-	 cur_max_pow = cur_max_pow + 1
+   -- Add any remaining terms, up to OCTAVES.
+   --
+   if octaves then
+      while cur_octave <= octaves do
+	 add_term (cur_octave, 1)
+	 cur_octave = cur_octave + 1
       end
    end
 
-   if const_factor_sum ~= 1 then
+   if const_factor_sum ~= 1
+      and (type (params) ~= 'table'
+	or params.auto_scale ~= false)
+   then
       sum = sum * (1 / const_factor_sum)
    end
 
@@ -1354,7 +1410,139 @@ end
 -- explanation of PARAMS.
 --
 function perlin_abs_series_tex (params)
-   return fourier_series_tex (abs_tex (perlin_tex()), params)
+   return fourier_series_tex (perlin_abs_tex(), params)
+end
+
+
+----------------------------------------------------------------
+-- array functions
+
+-- Use binary search to locate the entries in TABLE which are closest
+-- to VAL.  SUB_KEY is a key for each table entry; if non-nil it is
+-- used to retrieve the key from each entry for comparison (otherwise
+-- the entries themselves are used).
+--
+-- Two indices are returned:  LO and HI; HI always equals LO+1 unless
+-- there are less than two entries in TABLE (if there are no entries,
+-- then LO and HI will be 1 and 0; if there is one entry then LO and
+-- HI will both be 1).  If VAL exactly matches an entry, then
+-- TABLE[LO] will be that entry.  If VAL lies between two entries,
+-- then LO and HI will be the indices of entries on either side of it.
+-- Otherwise, LO and HI will be the indices of the two entries closest
+-- to it (the two lowest entries in the table, or the two highest).
+--
+function binary_search (table, val, sub_key)
+   local lo, hi = 1, #table
+
+   while lo < hi - 1 do
+      local mid = math.floor ((lo + hi) / 2)
+
+      local entry = table[mid]
+      if sub_key then
+	 entry = entry[sub_key]
+      end
+
+      if entry > val then
+	 hi = mid
+      else
+	 lo = mid
+      end
+   end
+
+   return lo, hi
+end
+
+-- Lookup VAL in TABLE, using linear interpolation to compute entries
+-- not explicitly present.
+--
+-- TABLE should be a sequential array of entries like {VAL1, VAL2, ...},
+-- and be sorted by the values of VAL1.
+--
+function linear_interp_lookup (table, val)
+
+   -- If there's only one entry in the table, just return it.
+   --
+   if #table == 1 then
+      local entry = table[1]
+      if entry[1] == val then
+	 return entry
+      else
+	 -- Copy ENTRY so we can use the correct value for ENTRY[1]
+	 local new_entry = {val}
+	 for i = 2, #entry do
+	    new_entry[i] = entry[i]
+	 end
+	 return new_entry
+      end
+   end
+
+   -- Use binary search to locate the nearest entries to VAL.
+   --
+   local lo, hi = binary_search (table, val, 1)
+   lo = table[lo]
+   hi = table[hi]
+
+   if lo[1] == val then
+      -- found exact match
+      return lo
+   else
+      -- use interpolation
+
+      local interp = {}
+
+      local val_diff = val - lo[1]
+
+      local inv_denom = 1 / (hi[1] - lo[1])
+
+      interp[1] = val
+      
+      for i = 2, #lo do
+	 local slope = (hi[i] - lo[i]) * inv_denom
+	 interp[i] = lo[i] + val_diff * slope
+      end
+
+      return interp
+   end
+end
+
+
+----------------------------------------------------------------
+--
+-- Filename manipulation
+
+-- Return the directory portion of FILENAME, or nil if it has none
+--
+function filename_dir (filename)
+   return string.match (filename, "^(.*)/[^/]*$")
+end
+
+-- Return the extension (last part following a period) of FILENAME, or
+-- nil if it has none.
+--
+function filename_ext (filename)
+   return string.match (filename, "[.]([^./]*)$")
+end
+
+-- If FILENAME is relative, return it appended to DIR, otherwise just
+-- return FILENAME.  If DIR is nil, then just return FILENAME.
+--
+function filename_in_dir (filename, dir)
+   if dir and not string.match (filename, "^/") then
+      filename = dir.."/"..filename
+   end
+   return filename
+end
+
+-- If FILENAME has DIR as a prefix, followed by a directory separator,
+-- then return the portion of FILENAME following the directory
+-- separator; otherwise, just return FILENAME.
+-- 
+function filename_rel (filename, dir)
+   if string.sub (filename, 1, #dir + 1) == dir.."/" then
+      return string.sub (filename, #dir + 2)
+   else
+      return filename
+   end
 end
 
 
@@ -1364,14 +1552,6 @@ end
 
 include_path = { "." }
 
-
-function filename_dir (filename)
-   return string.match (filename, "^(.*)/[^/]*$")
-end
-function filename_ext (filename)
-   return string.match (filename, "[.]([^./]*)$")
-end
-
 function load_include (filename)
    local loaded, loaded_filename, err_msg
 
@@ -1536,6 +1716,27 @@ function use (filename)
 end
 
 
+-- Return a string containing the entire contents of the file
+-- FILENAME.  If an error occurs, return nil and an error description.
+--
+-- This is basically equivalent to io.open(filename,"r"):read"*a"
+-- (plus closing the file) but much more efficient and less likely to
+-- thrash the system to death when reading huge files.
+--
+function read_file (filename)
+   local contents = raw.read_file (filename)
+   if not contents then
+      local stream, err = io.open (filename, "r")
+      if not stream then
+	 return nil, err
+      end
+      contents = stream:read ("*a")
+      stream:close ()
+   end
+   return contents
+end
+
+
 ----------------------------------------------------------------
 --
 -- Autoloading
@@ -1594,7 +1795,7 @@ local scene_loaders = {}
 -- handled by the C++ core.  To load any supported format, use the
 -- scene "load" method.
 --
-function load_scene (filename, fmt, rscene, rcamera, ...)
+function load_scene (filename, fmt, rscene, rcamera, rparams, ...)
    local loader = scene_loaders[fmt]
 
    if loader then
@@ -1606,9 +1807,13 @@ function load_scene (filename, fmt, rscene, rcamera, ...)
       --
       camera = rcamera
 
+      -- and params table
+      --
+      params = rparams
+
       -- Call the loader.
       --
-      return loader (filename, scene, camera, ...)
+      return loader (filename, scene, camera, params, ...)
    else
       return false
    end
@@ -1694,6 +1899,7 @@ scene_loaders.luac = scene_loaders.lua
 scene_loaders.luo = scene_loaders.lua
 
 add_autoload_stub (scene_loaders, "nff", "load-nff.lua", "load_nff")
+add_autoload_stub (scene_loaders, "pbrt", "load-pbrt.lua", "load_pbrt")
 
 
 -- arch-tag: e5dc4da4-c3f0-45e7-a4a1-a20cb4db6d6b
