@@ -12,7 +12,6 @@
 
 #include "surface.h"
 #include "light.h"
-#include "shadow-ray.h"
 
 #include "space.h"
 
@@ -70,22 +69,18 @@ Space::intersect (Ray &ray, RenderContext &context) const
   return closest_isec_cb.closest;
 }
 
+
 
 // Simple (boolean) intersection testing
 
 struct IntersectsCallback : Space::IntersectCallback
 {
-  IntersectsCallback (const ShadowRay &_ray, RenderContext &_context,
-		      const Surface *_reject = 0)
-    : ray (_ray), intersects (false),
-      context (_context), reject (_reject)
+  IntersectsCallback (const Ray &_ray, RenderContext &_context)
+    : ray (_ray), intersects (false), context (_context)
   { }
 
   virtual bool operator() (const Surface *surf)
   {
-    if (surf == reject)
-      return false;
-
     intersects = surf->intersects (ray, context);
 
     if (intersects)
@@ -96,31 +91,22 @@ struct IntersectsCallback : Space::IntersectCallback
     return intersects;
   }
 
-  const ShadowRay &ray;
+  const Ray &ray;
 
   // True if we found an intersecting object.
   //
   bool intersects;
 
   RenderContext &context;
-
-  // If non-zero, this surface is always immediately rejected.
-  //
-  const Surface *reject;
 };
 
 
 // Return true if any object intersects RAY.
 //
 bool
-Space::intersects (const ShadowRay &ray, RenderContext &context) const
+Space::intersects (const Ray &ray, RenderContext &context) const
 {
-  // If possible, prime the negative intersect cache with the current
-  // surface, to avoid wasting time test it for intersection.
-  //
-  const Surface *reject = ray.isec.no_self_shadowing;
-
-  IntersectsCallback intersects_cb (ray, context, reject);
+  IntersectsCallback intersects_cb (ray, context);
 
   for_each_possible_intersector (ray, intersects_cb, context,
 				 context.stats.shadow);
@@ -129,4 +115,74 @@ Space::intersects (const ShadowRay &ray, RenderContext &context) const
 }
 
 
+
+// Occludes calculation, including partial occlusion.
+
+struct OccludesCallback : Space::IntersectCallback
+{
+  OccludesCallback (const Ray &_ray, const Medium &_medium,
+		    Color &_total_transmittance,
+		    RenderContext &_context)
+    : ray (_ray), total_transmittance (_total_transmittance),
+      medium (_medium), context (_context), occludes (false)
+  { }
+
+  virtual bool operator() (const Surface *surf)
+  {
+    occludes = surf->occludes (ray, medium, total_transmittance, context);
+    if (occludes)
+      stop_iteration ();
+    return occludes;
+  }
+
+  const Ray &ray;
+
+  // Product of all surfaces encountered so far.
+  //
+  Color &total_transmittance;
+
+  // Medium in which to evaluate material occlusion.
+  //
+  const Medium &medium;
+
+  RenderContext &context;
+
+  // True if we found a totally-occluding object.
+  //
+  bool occludes;
+};
+
+
+// Return true if some surface in this space completely occludes RAY.
+// If no surface completely occludes RAY, then return false, and
+// multiply TOTAL_TRANSMITTANCE by the transmittance of any surfaces
+// in this space which partially occlude RAY, evaluated in medium
+// MEDIUM.
+//
+// Note that this method does not try to handle non-trivial forms of
+// transparency/translucency (for instance, a "glass" material is
+// probably considered opaque because it changes light direction as
+// well as transmitting it), nor does it deal with anything except
+// surfaces.
+//
+// [This interface is slight awkward for reasons of speed --
+// returning and checking for a boolean value for common cases is
+// significantly faster than, for instance, a simple "transmittance"
+// method, which requires handling Color values for all cases.]
+//
+bool
+Space::occludes (const Ray &ray, const Medium &medium,
+		 Color &total_transmittance,
+		 RenderContext &context)
+  const
+{
+  OccludesCallback occludes_cb (ray, medium, total_transmittance, context);
+
+  for_each_possible_intersector (ray, occludes_cb, context,
+				 context.stats.shadow);
+
+  return occludes_cb.occludes;
+}
+
+
 // arch-tag: 550f9905-7373-4008-9c4e-e939d931f01d
