@@ -10,11 +10,13 @@
 // Written by Miles Bader <miles@gnu.org>
 //
 
+#include <stdexcept>
+
 #include "snogmath.h"
 #include "cos-dist.h"
 #include "intersect.h"
 #include "sphere-isec.h"
-#include "sample-sphere.h"
+#include "sphere-sample.h"
 
 #include "sphere-light.h"
 
@@ -22,6 +24,16 @@
 using namespace snogray;
 
 
+SphereLight::SphereLight (const Pos &_pos, float _radius,
+			  const TexVal<Color> &_intensity)
+  : pos (_pos), radius (_radius), intensity (_intensity.default_val) 
+{
+  if (_intensity.tex)
+    throw std::runtime_error
+      ("textured intensity not supported by SphereLight");
+}
+
+
 // Return the solid angle subtended by this light, where LIGHT_CENTER_VEC
 // is a vector from the viewer to the light's center.
 //
@@ -89,9 +101,11 @@ SphereLight::sample (const Intersect &isec, const UV &param) const
 	  // The "real" distance must terminate at the surface of the
 	  // sphere, so we need to do that calculation too...
 	  //
-	  dist_t dist = sphere_intersect (radius, -light_center_vec, s_dir);
-
-	  return Sample (intensity, pdf, s_dir, dist);
+	  dist_t dist;
+	  if (sphere_intersects (Pos(0,0,0), radius,
+				 Pos (-light_center_vec), s_dir,
+				 dist))
+	    return Sample (intensity, pdf, s_dir, dist);
 	}
     }
 
@@ -108,7 +122,7 @@ SphereLight::sample (const UV &param, const UV &dir_param) const
 {
   // Sample position on sphere's surface.
   //
-  Vec s_pos_vec = sample_sphere (param);
+  Vec s_pos_vec = sphere_sample (param);
   Pos s_pos = pos + s_pos_vec * radius;
   float pos_pdf = 1 / (radius*radius * 4 * PIf);   // 1 / area
 
@@ -158,9 +172,10 @@ SphereLight::eval (const Intersect &isec, const Vec &dir) const
   //
   if (light_center_vec.z >= -radius)
     {
-      float dist = sphere_intersect (radius, -light_center_vec, dir);
-
-      if (dist)
+      float dist;
+      if (sphere_intersects (Pos (0,0,0), radius,
+			     Pos (-light_center_vec), dir,
+			     dist))
 	{
 	  // The distribution used here is constant over a solid angle when
 	  // viewed by an external observer, meaning that it also has a
