@@ -1,6 +1,6 @@
 // surface-group.cc -- Group of surfaces
 //
-//  Copyright (C) 2007, 2008  Miles Bader <miles@gnu.org>
+//  Copyright (C) 2007, 2008, 2010  Miles Bader <miles@gnu.org>
 //
 // This source code is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -47,3 +47,17 @@ SurfaceGroup::add_to_space (SpaceBuilder &space_builder) const
        si != surfaces.end(); ++si)
     (*si)->add_to_space (space_builder);
 }
+
+
+// If this surface, or some part of it, uses any light-emitting
+// materials, add appropriate Light objects to LIGHTS.  Any lights
+// added become owned by the owner of LIGHTS, and will be destroyed
+// when it is.
+//
+void
+SurfaceGroup::add_lights (std::vector<Light *> &lights) const
+{
+  for (std::vector<const Surface *>::const_iterator si = surfaces.begin();
+       si != surfaces.end(); ++si)
+    (*si)->add_lights (lights);
+}
