@@ -13,7 +13,7 @@
 #ifndef __UV_IO_H__
 #define __UV_IO_H__
 
-#include <fstream>
+#include <iosfwd>
 
 #include "uv.h"
 
