@@ -13,7 +13,7 @@
 #ifndef __XFORM_IO_H__
 #define __XFORM_IO_H__
 
-#include <fstream>
+#include <iosfwd>
 
 #include "xform.h"
 
